### 部署
#### 1、安装软件
- jdk1.8
- mysql8/达梦8/Oracle12c+
- redis5+
#### 2、执行sql脚本
在 `/sql` 文件夹中执行sql脚本
#### 3、修改配置文件
在 `flow-app/src/main/resources` 文件夹中修改`.yml`结尾的配置文件。
```yml
spring:
  datasource:
    driver-class-name: com.mysql.cj.jdbc.Driver # mysql驱动包
    type: com.zaxxer.hikari.HikariDataSource
    url: jdbc:mysql://你的服务器id:端口/flow?characterEncoding=utf8&useUnicode=true&serverTimezone=Asia/Shanghai&nullCatalogMeansCurrent=true
    username: 数据库账号
    password: 数据库密码
  redis:
    timeout: 6000ms
    database: 0 
    host: 你的服务器id
    port: 6379
```
- 1.配置mysql的服务器ip地址与端口
- 2.配置redis的服务器ip地址与端口
#### 4、部署
编译后修改`flow-app/Dockerfile`文件中合适的端口后使用docker执行该文件即可完成部署。

如果不采用docker部署只需安装好jdk环境，然后通过一下命令即可执行程序
```shell
java -jar 程序名.jar
```
### 适配达梦数据库

#### 1、添加达梦数据库驱动

```xml

<dependency>
    <groupId>com.dameng</groupId>
    <artifactId>DmJdbcDriver18</artifactId>
</dependency>
```

#### 2、数据库连接配置

url后缀添加 **compatibleMode=oracle**

```yml
spring:
  datasource:
    driver-class-name: dm.jdbc.driver.DmDriver
    type: com.zaxxer.hikari.HikariDataSource
    url: jdbc:dm://127.0.0.1:30236?compatibleMode=oracle
    username: SYSDBA
    password: SYSDBA001
```

#### 3、主键生成配置
| id生成 | 实现                                                             |
|------|----------------------------------------------------------------|
| 自增ID | @KeySequence(value = "SEQ_MODEL_STRING_KEY", dbType = DbType.DM) 
|
| 雪花算法 | @TableId(type = IdType.ASSIGN_ID)                              |
#### 4、分页方言配置
在 ***com.flow.common.mybatis.configure.MybatisPlusConfigure***文件中，修改为***DbType.DM***
```java
    @Bean
    public MybatisPlusInterceptor mybatisPlusInterceptor() {
        MybatisPlusInterceptor mybatisPlusInterceptor = new MybatisPlusInterceptor();
        // 分页插件
        mybatisPlusInterceptor.addInnerInterceptor(new PaginationInnerInterceptor(DbType.DM));
        return mybatisPlusInterceptor;
    }
```
#### 5、修改递归查询方言
项目中的查询使用到mysql8的***with recursive*** 递归查询，修改为***CONNECT BY*** 递归查询
#### 6、执行sql脚本
***sql/dm.sql***