package com.flow.common.core.util;


import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.net.InetAddress;
import java.net.UnknownHostException;

public class RequestUtils {
    private static final Logger log = LoggerFactory.getLogger(RequestUtils.class);
    private static final String UNKNOWN = "unknown";
    private static final String LOCALHOST_IP = "0:0:0:0:0:0:0:1";
    private static final String LOCALHOST_IP1 = "127.0.0.1";

    public static HttpServletRequest getRequest() {
        return ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
    }

    public static String getIp() {
        HttpServletRequest request = RequestUtils.getRequest();
        // 获取请求主机IP地址,如果通过代理进来，则透过防火墙获取真实IP地址
        String ip = request.getHeader("X-Original-Forwarded-For");
        if (StringUtils.isEmpty(ip) || UNKNOWN.equalsIgnoreCase(ip)) {
            ip = request.getHeader("X-Forwarded-For");
        }
        if (StringUtils.isEmpty(ip) || UNKNOWN.equalsIgnoreCase(ip)) {
            ip = request.getHeader("x-forwarded-for");
        }
        if (StringUtils.isEmpty(ip) || UNKNOWN.equalsIgnoreCase(ip)) {
            ip = request.getHeader("Proxy-Client-IP");
        }
        if (StringUtils.isEmpty(ip) || UNKNOWN.equalsIgnoreCase(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (StringUtils.isEmpty(ip) || UNKNOWN.equalsIgnoreCase(ip)) {
            ip = request.getHeader("HTTP_CLIENT_IP");
        }
        if (StringUtils.isEmpty(ip) || UNKNOWN.equalsIgnoreCase(ip)) {
            ip = request.getHeader("HTTP_X_FORWARDED_FOR");
        }
        if (StringUtils.isEmpty(ip) || UNKNOWN.equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
            if (LOCALHOST_IP1.equals(ip) || LOCALHOST_IP.equals(ip)) {
                // 获取本机真正的ip地址
                try {
                    ip = InetAddress.getLocalHost().getHostAddress();
                } catch (UnknownHostException e) {
                    log.error("getClientIp error: {}", e.getMessage());
                }
            }
        }
        if (!StringUtils.isEmpty(ip) && ip.indexOf(",") > 0) {
            ip = ip.substring(0, ip.indexOf(","));
        }
        return LOCALHOST_IP.equals(ip) ? LOCALHOST_IP1 : ip;
    }

    public static String getUserAgent() {
        HttpServletRequest request = RequestUtils.getRequest();
        return request.getHeader("User-Agent");
    }

    public static String getBrowser() {
        HttpServletRequest request = RequestUtils.getRequest();
        String uaStr = request.getHeader("User-Agent");
        if (StringUtils.isEmpty(uaStr)) {
            return "未知";
        }
        String browser = "";
        if (uaStr.contains("MSIE") || uaStr.contains("Trident")) {
            browser = "IE";
        } else if (uaStr.contains("Firefox")) {
            browser = "Firefox";
        } else if (uaStr.contains("Chrome")) {
            browser = "Chrome";
        } else if (uaStr.contains("Opera")) {
            browser = "Opera";
        } else if (uaStr.contains("Safari")) {
            browser = "Safari";
        } else {
            browser = "未知";
        }
        return browser;
    }

    public static String getOs() {
        HttpServletRequest request = RequestUtils.getRequest();
        String uaStr = request.getHeader("User-Agent");
        if (StringUtils.isEmpty(uaStr)) {
            return "未知";
        }
        String system = "";
        if (uaStr.contains("Windows NT 10.0")) {
            system = "Windows 10";
        } else if (uaStr.contains("Windows NT 6.3")) {
            system = "Windows 8.1";
        } else if (uaStr.contains("Windows NT 6.2")) {
            system = "Windows 8";
        } else if (uaStr.contains("Windows NT 6.1")) {
            system = "Windows 7";
        } else if (uaStr.contains("Windows NT 6.0")) {
            system = "Windows Vista";
        } else if (uaStr.contains("Windows NT 5.1")) {
            system = "Windows XP";
        } else if (uaStr.contains("Windows NT 5.0")) {
            system = "Windows 2000";
        } else if (uaStr.contains("Mac")) {
            system = "Mac";
        } else if (uaStr.contains("iPhone")) {
            system = "iPhone";
        } else if (uaStr.contains("iPad")) {
            system = "iPad";
        } else if (uaStr.contains("iPod")) {
            system = "iPod";
        } else if (uaStr.contains("Android")) {
            system = "Android";
        } else if (uaStr.contains("X11")) {
            system = "Unix";
        } else if (uaStr.contains("Linux")) {
            system = "Linux";
        } else {
            system = "其他";
        }
        return system;
    }

}
