package com.flow.flowable.behavior;

import com.flow.flowable.cmd.CopyProcessInstanceCmd;
import com.flow.flowable.event.CustomFlowableEngineEvent;
import com.flow.flowable.event.CustomFlowableEventType;
import org.flowable.common.engine.api.delegate.Expression;
import org.flowable.common.engine.api.delegate.event.FlowableEventDispatcher;
import org.flowable.engine.ManagementService;
import org.flowable.engine.delegate.DelegateExecution;
import org.flowable.engine.impl.bpmn.behavior.AbstractBpmnActivityBehavior;
import org.flowable.engine.impl.cfg.ProcessEngineConfigurationImpl;
import org.flowable.engine.impl.persistence.entity.*;
import org.flowable.engine.impl.util.CommandContextUtil;
import org.flowable.http.common.impl.ExpressionUtils;

import java.util.List;
import java.util.Objects;

public class CopyActivityBehavior extends AbstractBpmnActivityBehavior {
    private static final long serialVersionUID = 1L;
    protected Expression assignee;

    @Override
    public void execute(DelegateExecution execution) {
        String assignee = ExpressionUtils.getStringFromField(this.assignee, execution);
        ActivityInstanceEntityManager activityInstanceEntityManager = CommandContextUtil.getActivityInstanceEntityManager();
        List<ActivityInstanceEntity> activityInstanceEntities = activityInstanceEntityManager.findActivityInstancesByExecutionAndActivityId(execution.getId(), execution.getCurrentActivityId());
        for (ActivityInstanceEntity activityInstanceEntity : activityInstanceEntities) {
            activityInstanceEntity.setAssignee(assignee);
            activityInstanceEntityManager.update(activityInstanceEntity);
        }
        HistoricActivityInstanceEntityManager historicActivityInstanceEntityManager = CommandContextUtil.getHistoricActivityInstanceEntityManager();
        List<HistoricActivityInstanceEntity> historicActivityInstanceEntities = historicActivityInstanceEntityManager.findHistoricActivityInstancesByExecutionAndActivityId(execution.getId(), execution.getCurrentActivityId());
        for (HistoricActivityInstanceEntity historicActivityInstanceEntity : historicActivityInstanceEntities) {
            historicActivityInstanceEntity.setAssignee(assignee);
            historicActivityInstanceEntityManager.update(historicActivityInstanceEntity);
        }
        ProcessEngineConfigurationImpl processEngineConfiguration = CommandContextUtil.getProcessEngineConfiguration();
        ManagementService managementService = processEngineConfiguration.getManagementService();
        managementService.executeCommand(new CopyProcessInstanceCmd(execution.getProcessInstanceId(), assignee));
        FlowableEventDispatcher eventDispatcher = processEngineConfiguration.getEventDispatcher();
        if (Objects.nonNull(eventDispatcher) && eventDispatcher.isEnabled()) {
            CustomFlowableEngineEvent customFlowableEngineEvent = new CustomFlowableEngineEvent((ExecutionEntityImpl) execution, CustomFlowableEventType.PROCESS_COPY);
            eventDispatcher.dispatchEvent(customFlowableEngineEvent, processEngineConfiguration.getEngineCfgKey());
        }
        leave(execution);
    }
}
