package com.flow.flowable.cmd;

import org.apache.commons.lang3.StringUtils;
import org.flowable.common.engine.impl.interceptor.Command;
import org.flowable.common.engine.impl.interceptor.CommandContext;
import org.flowable.engine.impl.util.CommandContextUtil;
import org.flowable.identitylink.service.HistoricIdentityLinkService;
import org.flowable.identitylink.service.impl.persistence.entity.HistoricIdentityLinkEntity;

import java.util.Date;
import java.util.List;

public class CopyProcessInstanceCmd implements Command<Void> {
    protected String processInstanceId;
    protected String assignee;
    protected String taskId;

    public CopyProcessInstanceCmd(String processInstanceId, String assignee) {
        this.processInstanceId = processInstanceId;
        this.assignee = assignee;
    }

    public CopyProcessInstanceCmd(String processInstanceId, String taskId, String assignee) {
        this.processInstanceId = processInstanceId;
        this.taskId = taskId;
        this.assignee = assignee;
    }

    @Override
    public Void execute(CommandContext commandContext) {
        HistoricIdentityLinkService historicIdentityLinkService = CommandContextUtil.getProcessEngineConfiguration(commandContext)
                .getIdentityLinkServiceConfiguration()
                .getHistoricIdentityLinkService();
        List<HistoricIdentityLinkEntity> historicIdentityLinkEntityList = historicIdentityLinkService.findHistoricIdentityLinksByProcessInstanceId(processInstanceId);
        boolean noneMatch = historicIdentityLinkEntityList
                .stream()
                .noneMatch(historicIdentityLinkEntity -> "copy".equals(historicIdentityLinkEntity.getType()) && historicIdentityLinkEntity.getUserId().equals(assignee));
        if (noneMatch) {
            HistoricIdentityLinkEntity historicIdentityLink = historicIdentityLinkService.createHistoricIdentityLink();
            historicIdentityLink.setType("copy");
            historicIdentityLink.setUserId(assignee);
            historicIdentityLink.setCreateTime(new Date());
            historicIdentityLink.setProcessInstanceId(processInstanceId);
            if (StringUtils.isNotBlank(taskId)) {
                historicIdentityLink.setTaskId(taskId);
            }
            historicIdentityLinkService.insertHistoricIdentityLink(historicIdentityLink, false);
        }
        return null;
    }
}
