package com.flow.flowable.cmd;

import org.flowable.bpmn.model.MultiInstanceLoopCharacteristics;
import org.flowable.common.engine.api.FlowableException;
import org.flowable.common.engine.impl.interceptor.Command;
import org.flowable.common.engine.impl.interceptor.CommandContext;
import org.flowable.engine.ManagementService;
import org.flowable.engine.RuntimeService;
import org.flowable.engine.TaskService;
import org.flowable.engine.impl.cfg.ProcessEngineConfigurationImpl;
import org.flowable.engine.impl.persistence.entity.ActivityInstanceEntity;
import org.flowable.engine.impl.persistence.entity.ActivityInstanceEntityManager;
import org.flowable.engine.impl.util.CommandContextUtil;
import org.flowable.engine.runtime.Execution;
import org.flowable.task.api.Task;

import java.util.List;
import java.util.Objects;

public class DeleteTaskExecutionCmd implements Command<Void> {
    protected Task task;
    protected String assignee;

    public DeleteTaskExecutionCmd(Task task, String assignee) {
        this.task = task;
        this.assignee = assignee;
    }

    @Override
    public Void execute(CommandContext commandContext) {
        ProcessEngineConfigurationImpl processEngineConfiguration = CommandContextUtil.getProcessEngineConfiguration(commandContext);
        ManagementService managementService = processEngineConfiguration.getManagementService();
        MultiInstanceLoopCharacteristics loopCharacteristics = managementService.executeCommand(new GetLoopCharacteristicsCmd(this.task.getProcessInstanceId(), this.task.getTaskDefinitionKey()));
        if (Objects.nonNull(loopCharacteristics)) {
            ActivityInstanceEntityManager activityInstanceEntityManager = processEngineConfiguration.getActivityInstanceEntityManager();
            RuntimeService runtimeService = processEngineConfiguration.getRuntimeService();
            TaskService taskService = processEngineConfiguration.getTaskService();
            String collectionName = loopCharacteristics.getInputDataItem();
            collectionName = collectionName.substring(2, collectionName.length() - 1);
            List<String> assigneeList = runtimeService.getVariable(
                    this.task.getProcessInstanceId(),
                    collectionName,
                    List.class
            );
            if (!assigneeList.contains(this.assignee)) {
                throw new FlowableException(String.format("会签办理者中没有用户：%s", this.assignee));
            }
            if (assigneeList.size() == 1) {
                throw new FlowableException("会签必须保留一个用户");
            }
            Execution miExecution = managementService.executeCommand(new GetMultiInstanceRootExecutionCmd(this.task.getProcessInstanceId(), this.task.getTaskDefinitionKey()));
            if (loopCharacteristics.isSequential()) {
                Task deleteTask = taskService.createTaskQuery()
                        .processInstanceId(this.task.getProcessInstanceId())
                        .taskDefinitionKey(this.task.getTaskDefinitionKey())
                        .taskAssignee(this.assignee)
                        .singleResult();
                if (Objects.nonNull(deleteTask)) {
                    throw new FlowableException("任务已开始减签失败");
                } else {
                    Integer nrOfInstances = runtimeService.getVariableLocal(miExecution.getId(), "nrOfInstances", Integer.class);
                    runtimeService.setVariableLocal(miExecution.getId(), "nrOfInstances", nrOfInstances - 1);
                }
            } else {
                List<Task> tasks = taskService.createTaskQuery()
                        .processInstanceId(this.task.getProcessInstanceId())
                        .taskDefinitionKey(this.task.getTaskDefinitionKey())
                        .active()
                        .list();
                if (tasks.stream().noneMatch(task -> task.getAssignee().equals(this.assignee))) {
                    throw new FlowableException(String.format("没有用户：%s的会签任务", this.assignee));
                }
                Integer nrOfActiveInstances = runtimeService.getVariableLocal(miExecution.getId(), "nrOfActiveInstances", Integer.class);
                runtimeService.setVariableLocal(miExecution.getId(), "nrOfActiveInstances", nrOfActiveInstances - 1);
                List<Task> deleteTasks = taskService.createTaskQuery()
                        .processInstanceId(this.task.getProcessInstanceId())
                        .taskDefinitionKey(this.task.getTaskDefinitionKey())
                        .taskAssignee(this.assignee)
                        .list();
                for (Task deleteTask : deleteTasks) {
                    runtimeService.deleteMultiInstanceExecution(deleteTask.getExecutionId(), false);
                    ActivityInstanceEntity activityInstanceEntity = activityInstanceEntityManager.findActivityInstanceByTaskId(deleteTask.getId());
                    managementService.executeCommand(new DeleteActivityEntityCmd(activityInstanceEntity.getId()));
                }
            }
            assigneeList.remove(this.assignee);
            runtimeService.setVariable(this.task.getProcessInstanceId(), collectionName, assigneeList);
        } else {
            throw new FlowableException("当前节点不支持减签");
        }
        return null;
    }
}
