package com.flow.flowable.converter;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.flow.flowable.model.CopyServiceTask;
import com.flow.flowable.model.JumpServiceTask;
import com.flow.flowable.model.MicroServiceTask;
import com.flow.flowable.model.MqServiceTask;
import com.flow.flowable.utils.ProcessElementUtil;
import org.flowable.bpmn.converter.ServiceTaskXMLConverter;
import org.flowable.bpmn.model.BaseElement;
import org.flowable.bpmn.model.BpmnModel;
import org.flowable.bpmn.model.ServiceTask;
import org.flowable.engine.impl.util.CommandContextUtil;

import javax.xml.stream.XMLStreamReader;
import java.util.Map;

public class CustomServiceTaskXMLConverter extends ServiceTaskXMLConverter {
    public CustomServiceTaskXMLConverter() {
    }


    @Override
    protected BaseElement convertXMLToElement(XMLStreamReader xtr, BpmnModel model) throws Exception {
        BaseElement baseElement = super.convertXMLToElement(xtr, model);
        if (baseElement instanceof ServiceTask) {
            ServiceTask serviceTask = (ServiceTask) baseElement;
            String serviceTaskType = serviceTask.getType();
            if ("copy".equals(serviceTaskType)) {
                CopyServiceTask copyServiceTask = new CopyServiceTask();
                copyServiceTask.setValues(serviceTask);
                this.convertCopyServiceTaskXMLProperties(copyServiceTask, model, xtr);
                return copyServiceTask;
            } else if ("jump".equals(serviceTaskType)) {
                JumpServiceTask jumpServiceTask = new JumpServiceTask();
                jumpServiceTask.setValues(serviceTask);
                this.convertJumpServiceTaskXMLProperties(jumpServiceTask, model, xtr);
                return jumpServiceTask;
            } else if ("micro".equals(serviceTaskType)) {
                MicroServiceTask microServiceTask = new MicroServiceTask();
                microServiceTask.setValues(serviceTask);
                this.convertMicroServiceTaskXMLProperties(microServiceTask, model, xtr);
                return microServiceTask;
            } else if ("mq".equals(serviceTaskType)) {
                MqServiceTask mqServiceTask = new MqServiceTask();
                mqServiceTask.setValues(serviceTask);
                this.convertMqServiceTaskXMLProperties(mqServiceTask, model, xtr);
            }
        }
        return baseElement;
    }

    protected void convertCopyServiceTaskXMLProperties(CopyServiceTask copyServiceTask, BpmnModel bpmnModel, XMLStreamReader xtr) throws Exception {
        String assignee = ProcessElementUtil.getFieldExtensionExpression(copyServiceTask, "assignee");
        copyServiceTask.setAssignee(assignee);
    }

    protected void convertJumpServiceTaskXMLProperties(JumpServiceTask jumpServiceTask, BpmnModel bpmnModel, XMLStreamReader xtr) throws Exception {
        String targetNode = ProcessElementUtil.getFieldExtensionValue(jumpServiceTask, "targetNode");
        jumpServiceTask.setTargetNode(targetNode);
    }

    protected void convertMicroServiceTaskXMLProperties(MicroServiceTask microServiceTask, BpmnModel bpmnModel, XMLStreamReader xtr) throws Exception {
        String serviceId = ProcessElementUtil.getFieldExtensionValue(microServiceTask, "serviceId");
        String url = ProcessElementUtil.getFieldExtensionValue(microServiceTask, "url");
        String method = ProcessElementUtil.getFieldExtensionValue(microServiceTask, "method");
        String headers = ProcessElementUtil.getFieldExtensionValue(microServiceTask, "headers");
        String params = ProcessElementUtil.getFieldExtensionValue(microServiceTask, "params");
        String body = ProcessElementUtil.getFieldExtensionValue(microServiceTask, "body");
        String responseVariableName = ProcessElementUtil.getFieldExtensionValue(microServiceTask, "responseVariableName");
        microServiceTask.setServiceId(serviceId);
        microServiceTask.setUrl(url);
        microServiceTask.setMethod(method);
        ObjectMapper objectMapper = CommandContextUtil.getProcessEngineConfiguration().getObjectMapper();
        microServiceTask.setHeaders(objectMapper.readValue(headers, new TypeReference<Map<String, Object>>() {}));
        microServiceTask.setParams(objectMapper.readValue(params, new TypeReference<Map<String, Object>>() {}));
        microServiceTask.setBody(objectMapper.readValue(body, new TypeReference<Map<String, Object>>() {}));
        microServiceTask.setResponseVariableName(responseVariableName);
    }

    protected void convertMqServiceTaskXMLProperties(MqServiceTask mqServiceTask, BpmnModel bpmnModel, XMLStreamReader xtr) throws Exception {
        String queue = ProcessElementUtil.getFieldExtensionValue(mqServiceTask, "queue");
        String params = ProcessElementUtil.getFieldExtensionValue(mqServiceTask, "params");
        mqServiceTask.setQueue(queue);
        ObjectMapper objectMapper = CommandContextUtil.getProcessEngineConfiguration().getObjectMapper();
        mqServiceTask.setParams(objectMapper.readValue(params, new TypeReference<Map<String, Object>>() {}));
    }
}
