package com.flow.flowable.loader;

import org.flowable.common.engine.api.FlowableException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.beans.factory.support.BeanDefinitionRegistryPostProcessor;
import org.springframework.core.Ordered;
import org.springframework.core.PriorityOrdered;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;


@Component
public class FlowableLoaderClass implements BeanDefinitionRegistryPostProcessor, PriorityOrdered {
    private static final Logger log = LoggerFactory.getLogger(FlowableLoaderClass.class);

    @Override
    public void postProcessBeanDefinitionRegistry(BeanDefinitionRegistry beanDefinitionRegistry) throws BeansException {
        List<String> serializableFlowableClassList = new ArrayList<>();
        serializableFlowableClassList.add("org.flowable.bpmn.model.BpmnModel");
        serializableFlowableClassList.add("org.flowable.bpmn.model.BaseElement");
        serializableFlowableClassList.add("org.flowable.bpmn.model.GraphicInfo");
        serializableFlowableClassList.add("org.flowable.common.engine.impl.event.FlowableEventSupport");
        serializableFlowableClassList.add("org.flowable.bpmn.model.BpmnDiEdge");
        serializableFlowableClassList.add("org.flowable.bpmn.model.ExtensionAttribute");
        serializableFlowableClassList.add("org.flowable.common.engine.impl.persistence.entity.AbstractEntity");
        serializableFlowableClassList.add("org.flowable.engine.impl.delegate.invocation.DefaultDelegateInterceptor");
        serializableFlowableClassList.add("org.flowable.common.engine.api.delegate.FlowableFunctionDelegate");
        serializableFlowableClassList.add("org.flowable.common.engine.impl.el.FlowableAstFunctionCreator");
        serializableFlowableClassList.add("org.flowable.common.engine.impl.el.function.AbstractFlowableVariableExpressionFunction");
        FlowableSerializable flowableSerializable = new FlowableSerializable();
        flowableSerializable.setSerializableFlowableClassList(serializableFlowableClassList);
        try {
            flowableSerializable.execute();
        } catch (Exception e) {
            log.error("Failed to make Flowable classes serializable", e);
            throw new FlowableException("Failed to make Flowable classes serializable", e);
        }
    }

    @Override
    public void postProcessBeanFactory(ConfigurableListableBeanFactory configurableListableBeanFactory) throws BeansException {

    }

    @Override
    public int getOrder() {
        return Ordered.LOWEST_PRECEDENCE;
    }
}
