package com.flow.flowable.validator;

import com.flow.flowable.utils.ProcessElementUtil;
import org.apache.commons.lang3.StringUtils;
import org.flowable.bpmn.model.FieldExtension;
import org.flowable.bpmn.model.Process;
import org.flowable.bpmn.model.ServiceTask;
import org.flowable.bpmn.model.TaskWithFieldExtensions;
import org.flowable.validation.ValidationError;
import org.flowable.validation.validator.impl.ServiceTaskValidator;

import java.util.List;

public class CustomServiceTaskValidator extends ServiceTaskValidator {

    @Override
    protected void verifyType(Process process, ServiceTask serviceTask, List<ValidationError> errors) {
        String type = serviceTask.getType();
        if (StringUtils.isNotEmpty(type)) {
            if (type.equalsIgnoreCase("copy")) {
                this.validateFieldDeclarationsForCopy(process, serviceTask, serviceTask.getFieldExtensions(), errors);
            } else if (type.equalsIgnoreCase("jump")) {
                this.validateFieldDeclarationsForJump(process, serviceTask, serviceTask.getFieldExtensions(), errors);
            } else if (type.equalsIgnoreCase("micro")) {
                this.validateFieldDeclarationsForMicro(process, serviceTask, serviceTask.getFieldExtensions(), errors);
            } else if (type.equalsIgnoreCase("mq")) {
                this.validateFieldDeclarationsForMq(process, serviceTask, serviceTask.getFieldExtensions(), errors);
            } else {
                super.verifyType(process, serviceTask, errors);
            }
        }
    }

    private void validateFieldDeclarationsForCopy(Process process, TaskWithFieldExtensions task, List<FieldExtension> fieldExtensions, List<ValidationError> errors) {
        String assignee = ProcessElementUtil.getFieldExtensionExpression(task, "assignee");
        if (StringUtils.isBlank(assignee)) {
            addError(errors, "flowable-copyTask-no-assignee", process, task, "抄送节点缺少assignee属性");
        }
    }

    private void validateFieldDeclarationsForJump(Process process, TaskWithFieldExtensions task, List<FieldExtension> fieldExtensions, List<ValidationError> errors) {
        String targetNode = ProcessElementUtil.getFieldExtensionValue(task, "targetNode");
        if (StringUtils.isBlank(targetNode)) {
            addError(errors, "flowable-jumpTask-no-targetNode", process, task, "跳转节点缺少targetNode属性");
        }
    }

    private void validateFieldDeclarationsForMicro(Process process, TaskWithFieldExtensions task, List<FieldExtension> fieldExtensions, List<ValidationError> errors) {
        String url = ProcessElementUtil.getFieldExtensionValue(task, "url");
        if (StringUtils.isBlank(url)) {
            addError(errors, "flowable-microTask-no-url", process, task, "微服务节点缺少url属性");
        }
        String method = ProcessElementUtil.getFieldExtensionValue(task, "method");
        if (StringUtils.isBlank(method)) {
            addError(errors, "flowable-microTask-no-method", process, task, "微服务节点缺少method属性");
        }
    }

    private void validateFieldDeclarationsForMq(Process process, TaskWithFieldExtensions task, List<FieldExtension> fieldExtensions, List<ValidationError> errors) {
        String queue = ProcessElementUtil.getFieldExtensionValue(task, "queue");
        if (StringUtils.isBlank(queue)) {
            addError(errors, "flowable-mqTask-no-queue", process, task, "MQ节点缺少queue属性");
        }
    }

}
