package com.flow.common.oauth2.configure;

import org.springframework.security.oauth2.common.util.OAuth2Utils;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.OAuth2Request;
import org.springframework.security.oauth2.provider.token.DefaultAuthenticationKeyGenerator;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.TreeSet;
import java.util.UUID;

/**
 * 重写token的key生成器
 */
public class AuthenticationKeyGenerator extends DefaultAuthenticationKeyGenerator {
    private static final String CLIENT_ID = "client_id";
    private static final String SCOPE = "scope";
    private static final String USERNAME = "username";

    @Override
    public String extractKey(OAuth2Authentication authentication) {
         Map<String, String> values = new LinkedHashMap<String, String>();
        OAuth2Request authorizationRequest = authentication.getOAuth2Request();
        if (!authentication.isClientOnly()) {
            values.put(USERNAME, authentication.getName());
        }
        values.put(CLIENT_ID, authorizationRequest.getClientId());
        if (authorizationRequest.getScope() != null) {
            values.put(SCOPE, OAuth2Utils.formatParameterList(new TreeSet<String>(authorizationRequest.getScope())));
        }
        // 添加uuid解决每次生成的 token都一样的问题
        values.put("uuid", UUID.randomUUID().toString());
        // 多地登录不同token可以设备设备id或ip地址
        // String deviceId = authorizationRequest.getRequestParameters().get("deviceId");
        // values.put("deviceId", deviceId);

        // 如果是多租户系统，这里要区分租户ID 条件
        // values.put("tenantId", "我是租户id");
        return generateKey(values);
    }

    // @Override
    // public String extractKey(OAuth2Authentication authentication) {
    //     Map<String, String> values = new LinkedHashMap<>();
    //     OAuth2Request authorizationRequest = authentication.getOAuth2Request();
    //     if (!authentication.isClientOnly()) {
    //         values.put(USERNAME, authentication.getName());
    //     }
    //     values.put(CLIENT_ID, authorizationRequest.getClientId());
    //     if (authorizationRequest.getScope() != null) {
    //         values.put(SCOPE, OAuth2Utils.formatParameterList(new TreeSet<>(authorizationRequest.getScope())));
    //     }
    //     values.put("login_type", (String) authorizationRequest.getRequestParameters().get("login_type"));
    //     values.put("uuid", UUID.randomUUID().toString());
    //     return generateKey(values);
    // }

}