package com.flow.common.oauth2.configure;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.autoconfigure.security.servlet.UserDetailsServiceAutoConfiguration;
import org.springframework.context.annotation.Bean;
import org.springframework.http.HttpMethod;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.oauth2.config.annotation.web.configuration.EnableResourceServer;
import org.springframework.security.oauth2.config.annotation.web.configuration.ResourceServerConfigurerAdapter;
import org.springframework.security.oauth2.config.annotation.web.configurers.ResourceServerSecurityConfigurer;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.security.web.access.AccessDeniedHandler;

import javax.servlet.http.HttpServletResponse;


@EnableResourceServer
@EnableGlobalMethodSecurity(prePostEnabled = true, proxyTargetClass = true, securedEnabled = true, jsr250Enabled = true)
@EnableAutoConfiguration(exclude = UserDetailsServiceAutoConfiguration.class)
public class ResourceServerConfigure extends ResourceServerConfigurerAdapter {
    @Autowired(required = false)
    private AccessDeniedHandler accessDeniedHandler;
    @Autowired(required = false)
    private AuthenticationEntryPoint authenticationEntryPoint;

    @Override
    public void configure(HttpSecurity http) throws Exception {
        http
                // 添加全局 CORS 配置
                //.cors().configurationSource(corsConfigurationSource()).and()
                .cors().and()
                .csrf().disable()

                // 跨域异常处理
                .exceptionHandling()
                .authenticationEntryPoint((request, response, authException) -> {
                    // 手动添加 CORS 头
                    response.setHeader("Access-Control-Allow-Origin", "*");
                    response.setHeader("Access-Control-Allow-Methods", "GET, POST, PUT, DELETE, OPTIONS");
                    response.setHeader("Access-Control-Allow-Headers", "Authorization, Content-Type");

                    // 返回 JSON 格式的错误信息
                    response.setContentType("application/json;charset=UTF-8");
                    response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
                    response.getWriter().write(
                            "{\"code\": 401, \"message\": \"Token 无效或已过期，请重新登录\"}"
                    );
                })
                .and()

                .sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS)
                .and()
                .authorizeRequests()
                .antMatchers(HttpMethod.OPTIONS, "/**").permitAll()
                .antMatchers("/static/file/**").permitAll()
                .antMatchers("/auth/login",
                        "/auth/code",
                        "/auth/logout",
                        "/auth/saltLogin",
                        "/auth/getTimeStamp",
                        "/user/list",
                        "/model/download",
                        "/user/list").permitAll()
                .antMatchers("/socket/**").permitAll()
                .anyRequest().authenticated()
                .and()
                .exceptionHandling()
                .accessDeniedHandler(accessDeniedHandler)
                .authenticationEntryPoint(authenticationEntryPoint);
    }

    @Override
    public void configure(ResourceServerSecurityConfigurer resources) {
        // 必须要要AuthorizationServer的资源编号保持一致
        resources.resourceId("flow-app");
        // 身份验证入口点
        if (authenticationEntryPoint != null) {
            resources.authenticationEntryPoint(authenticationEntryPoint);
        }
        // 拒绝访问处理程序
        if (accessDeniedHandler != null) {
            resources.accessDeniedHandler(accessDeniedHandler);
        }
    }

    // @Bean
    // public CorsConfigurationSource corsConfigurationSource() {
    //     CorsConfiguration configuration = new CorsConfiguration();
    //     // 设置允许的来源
    //     configuration.addAllowedOriginPattern("*");
    //     // 配置允许的HTTP方法
    //     configuration.addAllowedMethod("*");
    //     // 配置允许的请求头
    //     configuration.addAllowedHeader("*");
    //     // 允许凭证（如Cookie）跨域
    //     configuration.setAllowCredentials(true);
    //
    //     UrlBasedCorsConfigurationSource source = new UrlBasedCorsConfigurationSource();
    //     source.registerCorsConfiguration("/**", configuration);
    //     return source;
    // }

}