package com.flow.common.redis.configure;

import com.flow.common.redis.service.RedisService;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.core.RedisOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.GenericJackson2JsonRedisSerializer;
import org.springframework.data.redis.serializer.JdkSerializationRedisSerializer;
import org.springframework.data.redis.serializer.StringRedisSerializer;

@Configuration
public class LettuceRedisConfigure {

    @Bean(name = "redisTemplate")
    @ConditionalOnClass(RedisOperations.class)
    public <T> RedisTemplate<String, T> redisTemplate(RedisConnectionFactory factory) {
        RedisTemplate<String, T> redisTemplate = new RedisTemplate<>();
        //设置工厂
        redisTemplate.setConnectionFactory(factory);
        // Key采用String序列化
        redisTemplate.setKeySerializer(new StringRedisSerializer());
        // Hash的Key采用String序列化方式
        redisTemplate.setHashKeySerializer(new StringRedisSerializer());
        // Value采用jackson的json序列化方式
        GenericJackson2JsonRedisSerializer jackson2JsonRedisSerializer = new GenericJackson2JsonRedisSerializer();
        redisTemplate.setValueSerializer(jackson2JsonRedisSerializer);
        // HashValue采用jackson的json序列化方式
        redisTemplate.setHashValueSerializer(jackson2JsonRedisSerializer);
        // 关闭事务
        redisTemplate.setEnableTransactionSupport(false);
        //属性设置后
        redisTemplate.afterPropertiesSet();
        return redisTemplate;
    }

    @Bean(name = "jdkRedisTemplate")
    @ConditionalOnClass(RedisOperations.class)
    public <T> RedisTemplate<String, T> jdkRedisTemplate(RedisConnectionFactory factory) {
        RedisTemplate<String, T> redisTemplate = new RedisTemplate<>();
        redisTemplate.setConnectionFactory(factory);
        redisTemplate.setKeySerializer(new StringRedisSerializer());
        redisTemplate.setHashKeySerializer(new StringRedisSerializer());
        JdkSerializationRedisSerializer jdkSerializationRedisSerializer = new JdkSerializationRedisSerializer();
        redisTemplate.setValueSerializer(jdkSerializationRedisSerializer);
        redisTemplate.setHashValueSerializer(jdkSerializationRedisSerializer);
        redisTemplate.setEnableTransactionSupport(false);
        redisTemplate.afterPropertiesSet();
        return redisTemplate;
    }

    @Bean
    @ConditionalOnBean(name = "redisTemplate")
    public RedisService<?> redisService() {
        return new RedisService<>();
    }

}
