package com.flow.config;

import com.flow.constant.FileConstant;
import org.springframework.boot.web.servlet.MultipartConfigFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.util.unit.DataSize;
import org.springframework.web.servlet.config.annotation.CorsRegistry;
import org.springframework.web.servlet.config.annotation.ResourceHandlerRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import javax.servlet.MultipartConfigElement;
import java.io.File;

@Configuration
public class WebAppConfigurer implements WebMvcConfigurer {
    /**
     * 静态资源配置
     *
     * @param registry
     */
    @Override
    public void addResourceHandlers(ResourceHandlerRegistry registry) {
        String path = FileConstant.getPath();
        registry.addResourceHandler("/static/file/**").addResourceLocations("file:" + path).setCachePeriod(0);
        WebMvcConfigurer.super.addResourceHandlers(registry);
    }

    // @Override
    // public void addResourceHandlers(ResourceHandlerRegistry registry) {
    //     String path = FileConstant.getPath();
    //     // 保证路径以 / 或 \\ 结尾
    //     if (!path.endsWith(File.separator)) {
    //         path += File.separator;
    //     }
    //     registry.addResourceHandler("/static/file/**")
    //             .addResourceLocations("file:" + path)
    //             .setCachePeriod(0);
    // }

    @Override
    public void addCorsMappings(CorsRegistry registry) {
        registry
                .addMapping("/**")
                // 方案一：允许所有 origin（不带 credentials）
                .allowedOrigins("*")
                .allowedMethods("*")
                .allowedHeaders("*")
                // 方案二：指定 origin + 允许 credentials
                // .allowedOrigins("https://yourdomain.com")
                // .allowCredentials(true)
                // 允许的响应 Header
                .exposedHeaders("*")
                .maxAge(3600);
    }

    // @Override
    // public void addCorsMappings(CorsRegistry registry) {
    //     registry
    //             // 所有接口
    //             .addMapping("/**")
    //             // 允许跨域请求的域名
    //             .allowedOrigins("*")
    //             // 是否发送 Cookie
    //             .allowCredentials(true)
    //             // 所有请求方法
    //             .allowedMethods("*")
    //             // 允许的 Header
    //             .allowedHeaders("*")
    //             // 允许的响应 Header
    //             .exposedHeaders("*")
    //             .maxAge(3600);
    // }


    /**
     * 文件上传配置
     *
     * @return
     */
    @Bean
    public MultipartConfigElement multipartConfigElement() {
        MultipartConfigFactory factory = new MultipartConfigFactory();
        factory.setMaxFileSize(DataSize.ofBytes(50 * 1024 * 1024));
        factory.setMaxRequestSize(DataSize.ofBytes(1024 * 1024 * 1024));
        return factory.createMultipartConfig();
    }
}
