package com.flow.handler;

import com.flow.common.core.model.Result;
import org.springframework.http.HttpStatus;
import org.springframework.security.authentication.*;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.oauth2.client.resource.OAuth2AccessDeniedException;
import org.springframework.security.oauth2.common.exceptions.InvalidTokenException;
import org.springframework.security.oauth2.common.exceptions.UnapprovedClientAuthenticationException;
import org.springframework.security.oauth2.provider.error.OAuth2AuthenticationEntryPoint;
import org.springframework.stereotype.Component;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * 授权异常
 */
@Component
public class CustomAuthenticationEntryPoint extends OAuth2AuthenticationEntryPoint {
    @Override
    public void commence(HttpServletRequest request, HttpServletResponse response, AuthenticationException authException) throws IOException, ServletException {
        Throwable cause = authException.getCause();
        if (cause instanceof OAuth2AccessDeniedException) {
            Result.sendError(HttpStatus.FORBIDDEN.value(), "资源没有访问权限");
        } else if (cause instanceof UnapprovedClientAuthenticationException) {
            Result.sendError(HttpStatus.UNAUTHORIZED.value(), "未经批准的客户端身份验证");
        } else if (cause instanceof InvalidTokenException) {
            Result.sendError(HttpStatus.UNAUTHORIZED.value(), "登录已过期，请重新登录");
        } else if (cause instanceof InsufficientAuthenticationException) {
            Result.sendError(HttpStatus.UNAUTHORIZED.value(), "访问此资源需要完成身份验证");
        } else if (cause instanceof BadCredentialsException) {
            Result.sendError(HttpStatus.UNAUTHORIZED.value(), "客户端异常");
        } else if (cause instanceof UsernameNotFoundException) {
            Result.sendError(HttpStatus.NOT_FOUND.value(), "用户不存在");
        } else if (cause instanceof LockedException) {
            Result.sendError(HttpStatus.LOCKED.value(), "账户已锁定");
        } else if (cause instanceof AccountExpiredException) {
            Result.sendError(HttpStatus.FORBIDDEN.value(), "账户已过期");
        } else if (cause instanceof CredentialsExpiredException) {
            Result.sendError(HttpStatus.UNAUTHORIZED.value(), "凭据过期");
        } else if (cause instanceof DisabledException) {
            Result.sendError(HttpStatus.FORBIDDEN.value(), "账户已禁用");
        } else {
            Result.sendError(HttpStatus.FORBIDDEN.value(), "该资源没有访问权限");
        }
    }
}
