package com.flow.service.impl;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.flow.common.mybatis.service.impl.BaseServiceImpl;
import com.flow.dao.MenuDao;
import com.flow.entity.Menu;
import com.flow.mapstruct.MenuMapper;
import com.flow.model.MenuQuery;
import com.flow.service.MenuService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Comparator;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@Service
public class MenuServiceImpl extends BaseServiceImpl<MenuDao, Menu> implements MenuService {
    @Autowired
    private MenuDao menuDao;
    @Autowired
    private MenuMapper menuMapper;

    @Override
    public List<Menu> getTree(MenuQuery menuQuery) {
        List<Menu> list = menuDao.lambdaQueryChain()
                .like(menuQuery.getTitle() != null, Menu::getTitle, menuQuery.getTitle())
                .eq(menuQuery.getPid() != null, Menu::getPid, menuQuery.getPid())
                .orderByAsc(Menu::getSort)
                .list();
        // 构建菜单树
        return list.stream()
                .filter(m -> m.getPid() == 0)
                .sorted(Comparator.comparingInt(Menu::getSort))
                .peek(m -> m.setChildren(this.getChildNode(m, list)))
                .collect(Collectors.toList());
    }

    private List<Menu> getChildNode(Menu root, List<Menu> allListTree) {
        List<Menu> collect = allListTree.stream()
                .filter((treeEntity) -> treeEntity.getPid().equals(root.getId()))
                .sorted(Comparator.comparingInt(Menu::getSort))
                .peek((treeEntity) -> treeEntity.setChildren(getChildNode(treeEntity, allListTree)))
                .collect(Collectors.toList());
        if (CollectionUtils.isEmpty(collect)) {
            return null;
        }
        return collect;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Menu create(Menu menu) {
        menuDao.insert(menu);
        return menu;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Menu update(Menu menu) {
        menuDao.updateById(menu);
        return menu;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(Set<Long> ids) {
        menuDao.deleteBatchIds(ids);
    }
}
