package com.flow.service.impl;


import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.flow.common.core.exception.BaseException;
import com.flow.common.core.model.PageResult;
import com.flow.common.mybatis.constant.SqlConstant;
import com.flow.common.mybatis.service.impl.BaseServiceImpl;
import com.flow.dao.RoleDao;
import com.flow.entity.Menu;
import com.flow.entity.Role;
import com.flow.entity.User;
import com.flow.mapstruct.RoleMapper;
import com.flow.model.RoleQuery;
import com.flow.service.RoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;
import java.util.Set;

@Service
public class RoleServiceImpl extends BaseServiceImpl<RoleDao, Role> implements RoleService {
    @Autowired
    private RoleDao roleDao;
    @Autowired
    private RoleMapper roleMapper;

    @Override
    public PageResult<Role> getList(RoleQuery roleQuery) {
        Role role = roleMapper.toEntity(roleQuery);
        Page<Role> page = roleDao.lambdaQueryChain()
                .setEntity(role)
                .page(
                        new Page<>(roleQuery.getPage(), roleQuery.getLimit())
                );
        return new PageResult<>(page.getTotal(), page.getRecords());
    }

    @Override
    public List<Menu> getPermission(Long roleId) {
        return roleDao.getPermission(roleId);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Role create(Role role) {
        Optional<Role> optional = roleDao.lambdaQueryChain()
                .eq(Role::getName, role.getName())
                .last(SqlConstant.LIMIT1)
                .oneOpt();
        if (optional.isPresent()) {
            throw new BaseException("角色名已存在");
        }
        roleDao.insert(role);
        roleDao.setPermission(role.getId(), role.getPermissions());
        return role;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Role update(Role role) {
        roleDao.updateById(role);
        if (CollectionUtils.isNotEmpty(role.getPermissions())) {
            roleDao.delPermission(role.getId(), role.getPermissions());
            roleDao.setPermission(role.getId(), role.getPermissions());
        }
        List<Role> list = roleDao.lambdaQueryChain()
                .select(Role::getId)
                .eq(Role::getName, role.getName())
                .last(SqlConstant.LIMIT2)
                .list();
        if (list.size() > 1) {
            throw new BaseException("角色名已存在");
        }
        return role;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean delete(Set<Long> ids) {
        int deletedCount = roleDao.deleteBatchIds(ids);
        return deletedCount == ids.size();
    }

    @Override
    public List<User> getUsers(String roleId) {
        return roleDao.getUsers(roleId);
    }

    @Override
    public List<User> getUsers(List<String> roleIds) {
        return roleDao.getUsersByIds(roleIds);
    }
}
