package com.flow.controller;

import com.flow.common.core.model.Result;
import com.flow.entity.Dept;
import com.flow.model.DeptQuery;
import com.flow.service.DeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Set;

@RestController
@RequestMapping("/dept")
public class DeptController {
    @Autowired
    private DeptService deptService;

    @GetMapping("/{id}")
    public Result<Dept> get(@PathVariable Long id) {
        return Result.success(deptService.getById(id));
    }

    @PostMapping("/options")
    public Result<List<Dept>> getOptions(@RequestBody Set<String> deptIds) {
        List<Dept> options = deptService.getOptions(deptIds);
        return Result.success(options);
    }
    // @GetMapping
    // public Result<List<Dept>> getList(DeptQuery deptQuery) {
    //     return Result.success(deptService.getList(deptQuery));
    // }

    @GetMapping("/search")
    public Result<List<Dept>> search(DeptQuery deptQuery) {
        return Result.success(deptService.search(deptQuery));
    }

    @GetMapping("/child/{pid}")
    public Result<List<Dept>> getChild(@PathVariable Long pid) {
        return Result.success(deptService.getChild(pid));
    }

    @GetMapping("/tree")
    public Result<List<Dept>> getTree(DeptQuery deptQuery) {
        return Result.success(deptService.getTree(deptQuery));
    }
    @PreAuthorize("hasAnyAuthority('dept:add')")
    @PostMapping
    public Result<Dept> create(@RequestBody Dept dept) {
        return Result.success(deptService.create(dept));
    }

    @PreAuthorize("hasAnyAuthority('dept:edit')")
    @PutMapping
    public Result<Dept> update(@RequestBody Dept dept) {
        return Result.success(deptService.update(dept));
    }

    @PreAuthorize("hasAnyAuthority('dept:del')")
    @DeleteMapping
    public Result<?> delete(@RequestBody Set<Long> ids) {
        boolean success = deptService.delete(ids);
        if (success) {
            return Result.success();
        } else {
            return Result.error("删除失败!请联系管理员");
        }
    }
}
