package com.flow.controller;

import com.flow.common.core.model.PageResult;
import com.flow.common.core.model.Result;
import com.flow.entity.Menu;
import com.flow.entity.Role;
import com.flow.model.RoleQuery;
import com.flow.service.RoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Set;


@RestController
@RequestMapping("/role")
public class RoleController {
    @Autowired
    private RoleService roleService;

    @GetMapping("/{id}")
    public Result<Role> get(@PathVariable Long id) {
        return Result.success(roleService.getById(id));
    }

    @GetMapping
    public Result<PageResult<Role>> getList(RoleQuery roleQuery) {
        return Result.success(roleService.getList(roleQuery));
    }

    @GetMapping("/permission/{roleId}")
    public Result<List<Menu>> getPermission(@PathVariable Long roleId) {
        return Result.success(roleService.getPermission(roleId));
    }

    @PreAuthorize("hasAnyAuthority('role:add')")
    @PostMapping
    public Result<Role> create(@RequestBody Role role) {
        return Result.success(roleService.create(role));
    }

    @PreAuthorize("hasAnyAuthority('role:edit')")
    @PutMapping
    public Result<Role> update(@RequestBody Role role) {
        return Result.success(roleService.update(role));
    }

    @PreAuthorize("hasAnyAuthority('role:del')")
    @DeleteMapping
    public Result<?> delete(@RequestBody Set<Long> ids) {
        boolean success = roleService.delete(ids);
        if (success) {
            return Result.success();
        } else {
            return Result.error("删除失败!请联系管理员");
        }
    }
}
