package com.flow.service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.flow.common.mybatis.service.BaseService;
import com.flow.entity.DatasetField;
import com.flow.entity.FlowDefine;
import com.flow.entity.FlowModel;
import com.flow.entity.FlowModelHistory;
import com.flow.model.spreadsheet.SpreadSheet;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.List;
import java.util.Set;

public interface FlowModelService extends BaseService<FlowModel> {
    /**
     * 流程模型列表
     * @param flowModel
     * @return
     */
    List<FlowModel> list(FlowModel flowModel);

    /**
     * 转bpmn并下载
     * @param flowModel
     * @throws IOException
     */
    void downloadXml(FlowModel flowModel) throws IOException;

    /**
     * 创建模型
     * @param flowModel
     * @return
     */
    FlowModel create(FlowModel flowModel);

    /**
     * 更新模型
     * @param flowModel
     * @return
     */
    FlowModel update(FlowModel flowModel);

    /**
     * 删除模型
     * @param ids
     */
    void delete(Set<Long> ids);

    /**
     * 部署模型
     * @param modelId
     * @return
     */
    FlowDefine deploy(Long modelId) throws JsonProcessingException;

    /**
     * 挂起/启用
     * @param modelId
     * @return
     */
    FlowModel suspend(Long modelId);

    /**
     * 获取模型
     * @param modelCode
     * @return
     */
    FlowModel getModel(String modelCode);

    /**
     * 获取模型历史
     * @param modelCode
     * @return
     */
    List<FlowModelHistory> getModelHistory(String modelCode);

    /**
     * 切换历史模型
     * @param historyModelId
     */
    void switchModel(Long historyModelId);

    /**
     * 获取打印字段
     * @param modelId
     * @return
     */
    List<DatasetField> getReportField(String modelId);

    /**
     * 保存打印模板
     * @param spreadSheet
     * @param modelId
     * @return
     */
    SpreadSheet savePrint(SpreadSheet spreadSheet, String modelId);

    /**
     * 获取打印模板
     * @param modelId
     * @return
     */
    SpreadSheet getPrint(String modelId);

    void exportExcel(String modelId) throws UnsupportedEncodingException;
}
