package com.flow.config;

import org.flowable.common.engine.impl.persistence.deploy.DeploymentCache;
import org.flowable.engine.impl.persistence.deploy.ProcessDefinitionCacheEntry;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.Collection;
import java.util.Collections;
import java.util.Set;

@Component
public class CustomDeploymentCache implements DeploymentCache<ProcessDefinitionCacheEntry> {
    @Autowired
    private RedisTemplate<String, ProcessDefinitionCacheEntry> jdkRedisTemplate;
    private final String CachePrefix = "processDefineCache:";

    @Override
    public ProcessDefinitionCacheEntry get(String id) {
        return jdkRedisTemplate.opsForValue().get(this.CachePrefix + id);
    }

    @Override
    public boolean contains(String id) {
        Boolean has = jdkRedisTemplate.hasKey(this.CachePrefix + id);
        return has != null && has;
    }

    @Override
    public void add(String id, ProcessDefinitionCacheEntry processDefinitionCacheEntry) {
        jdkRedisTemplate.opsForValue().set(this.CachePrefix + id, processDefinitionCacheEntry);
    }

    @Override
    public void remove(String id) {
        jdkRedisTemplate.delete(this.CachePrefix + id);
    }

    @Override
    public void clear() {
        Set<String> keys = jdkRedisTemplate.keys(this.CachePrefix + "*");
        if (keys != null) {
            jdkRedisTemplate.delete(keys);
        }
    }

    @Override
    public Collection<ProcessDefinitionCacheEntry> getAll() {
        Set<String> keys = jdkRedisTemplate.keys(this.CachePrefix + "*");
        if (keys != null) {
            return jdkRedisTemplate.opsForValue().multiGet(keys);
        }
        return Collections.emptyList();
    }

    @Override
    public int size() {
        Set<String> keys = jdkRedisTemplate.keys(this.CachePrefix + "*");
        if (keys != null) {
            return keys.size();
        }
        return 0;
    }
}
