package com.flow.delegate;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.flow.enums.NotifyTypeEnum;
import com.flow.flowable.utils.ProcessElementUtil;
import lombok.extern.slf4j.Slf4j;
import org.flowable.bpmn.model.ServiceTask;
import org.flowable.engine.delegate.BpmnError;
import org.flowable.engine.delegate.DelegateExecution;
import org.flowable.engine.delegate.JavaDelegate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;


@Slf4j
@Component
public class NotifyDelegate implements JavaDelegate {
    @Autowired
    private SiteNotifyDelegate siteNotifyDelegate;
    // @Autowired
    // private MailNotifyDelegate mailNotifyDelegate;

    @Override
    public void execute(DelegateExecution execution) {
        ServiceTask notifyServiceTask = (ServiceTask) execution.getCurrentFlowElement();
        String types = ProcessElementUtil.getFieldExtensionValue(notifyServiceTask, "types");
        if (StringUtils.isBlank(types)) {
            throw new BpmnError("notify.error", "通知参数异常");
        }
        String[] typeArray = types.split(",");
        for (String type : typeArray) {
            NotifyTypeEnum typeEnum = NotifyTypeEnum.match(type);
            switch (typeEnum) {
                case SITE:
                    // 站内通知
                    siteNotifyDelegate.execute(execution);
                    break;
                case EMAIL:
                    // 邮件通知
                    //mailNotifyDelegate.execute(execution);
                    break;
                default:
                    break;
            }
        }
    }
}
