package com.flow.delegate;

import com.flow.entity.Notify;
import com.flow.enums.NotifyEnum;
import com.flow.flowable.utils.ProcessElementUtil;
import com.flow.service.NotifyService;
import lombok.extern.slf4j.Slf4j;
import org.flowable.bpmn.model.MultiInstanceLoopCharacteristics;
import org.flowable.bpmn.model.ServiceTask;
import org.flowable.common.engine.impl.el.ExpressionManager;
import org.flowable.engine.delegate.DelegateExecution;
import org.flowable.engine.delegate.JavaDelegate;
import org.flowable.engine.impl.cfg.ProcessEngineConfigurationImpl;
import org.flowable.http.common.impl.ExpressionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;

@Slf4j
@Component
public class SiteNotifyDelegate implements JavaDelegate {
    @Autowired
    private NotifyService notifyService;
    @Autowired
    private ProcessEngineConfigurationImpl processEngineConfiguration;

    @Override
    public void execute(DelegateExecution execution) {
        ServiceTask serviceTask = (ServiceTask) execution.getCurrentFlowElement();
        MultiInstanceLoopCharacteristics loopCharacteristics = serviceTask.getLoopCharacteristics();
        String receiver = execution.getVariableLocal(loopCharacteristics.getElementVariable(), String.class);
        ExpressionManager expressionManager = processEngineConfiguration.getExpressionManager();
        String subject = ProcessElementUtil.getFieldExtensionExpression(serviceTask, "subject");
        String content = ProcessElementUtil.getFieldExtensionExpression(serviceTask, "content");
        subject = ExpressionUtils.getStringFromField(expressionManager.createExpression(subject), execution);
        content = ExpressionUtils.getStringFromField(expressionManager.createExpression(content), execution);
        log.info("发送站内消息：{}", content);
        Notify notify = Notify.builder()
                .sender("admin")
                .type(NotifyEnum.SYSTEM)
                .receivingTime(LocalDateTime.now())
                .receiver(receiver)
                .subject(subject)
                .content(content)
                .build();
        notifyService.notify(notify);
    }
}
