package com.flow.listener;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.flow.entity.Notify;
import com.flow.enums.ApprovalTimeoutEnum;
import com.flow.enums.NotifyEnum;
import com.flow.flowable.utils.ProcessElementUtil;
import com.flow.service.FlowTaskService;
import com.flow.service.NotifyService;
import lombok.extern.slf4j.Slf4j;
import org.flowable.bpmn.model.BoundaryEvent;
import org.flowable.engine.TaskService;
import org.flowable.engine.delegate.DelegateExecution;
import org.flowable.engine.delegate.ExecutionListener;
import org.flowable.task.api.Task;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;

/**
 * 审批超时处理
 */
@Slf4j
@Component
public class ApprovalTimeoutListener implements ExecutionListener {
    @Autowired
    private FlowTaskService flowTaskService;
    @Autowired
    private TaskService taskService;
    @Autowired
    private NotifyService notifyService;

    @Override
    public void notify(DelegateExecution delegateExecution) {
        BoundaryEvent boundaryEvent = (BoundaryEvent) delegateExecution.getCurrentFlowElement();
        String action = ProcessElementUtil.getExtensionElementValue(boundaryEvent, "action");
        if (StringUtils.isNotBlank(action)) {
            List<Task> tasks = taskService.createTaskQuery()
                    .processInstanceId(delegateExecution.getProcessInstanceId())
                    .taskDefinitionKey(boundaryEvent.getAttachedToRefId())
                    .list();
            if (action.equals(ApprovalTimeoutEnum.AUTO_PASS.getAction())) {
                // 自动通过
                for (Task task : tasks) {
                    flowTaskService.autoComplete(task.getId());
                }
            } else if (action.equals(ApprovalTimeoutEnum.AUTO_REFUSE.getAction())) {
                // 自动拒绝
                for (Task task : tasks) {
                    flowTaskService.autoRefuse(task.getId());
                }
            } else if (action.equals(ApprovalTimeoutEnum.REMIND.getAction())) {
                // 提醒
                for (Task task : tasks) {
                    Notify notify = new Notify();
                    notify.setSender("admin");
                    notify.setType(NotifyEnum.SYSTEM);
                    notify.setReceivingTime(LocalDateTime.now());
                    notify.setReceiver(task.getAssignee());
                    notify.setUrl("/flow/todo");
                    notify.setSubject("审批提醒");
                    notify.setContent(String.format("您的审批任务【%s】已超时，请及时处理。", task.getName()));
                    notifyService.notify(notify);
                }
            }
        }
    }
}
