package com.flow.service.impl;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.flow.entity.FlowActivity;
import com.flow.entity.FlowDefine;
import com.flow.entity.FlowTask;
import com.flow.entity.node.ApprovalNode;
import com.flow.enums.ApprovalMultiEnum;
import com.flow.service.FlowActivityService;
import com.flow.service.FlowDefineService;
import com.flow.service.FlowTaskService;
import com.flow.service.UserService;
import org.flowable.engine.HistoryService;
import org.flowable.engine.RuntimeService;
import org.flowable.engine.history.HistoricActivityInstance;
import org.flowable.engine.history.HistoricProcessInstance;
import org.flowable.engine.runtime.ActivityInstance;
import org.flowable.engine.runtime.ProcessInstance;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class FlowActivityServiceImpl implements FlowActivityService {
    @Autowired
    private FlowDefineService flowDefineService;
    @Autowired
    private FlowTaskService flowTaskService;
    @Autowired
    private RuntimeService runtimeService;
    @Autowired
    private HistoryService historyService;
    @Autowired
    private UserService userService;

    @Override
    public List<FlowActivity> records(String instanceId) {
        ProcessInstance processInstance = runtimeService.createProcessInstanceQuery().processInstanceId(instanceId).singleResult();
        if (Objects.isNull(processInstance)) {
            return historyRecords(instanceId);
        }
        FlowDefine flowDefine = flowDefineService.getDefine(processInstance.getProcessDefinitionId());
        List<FlowTask> flowTasks = flowTaskService.lambdaQuery()
                .select(FlowTask::getId, FlowTask::getStatus)
                .eq(FlowTask::getInstanceId, instanceId)
                .list();
        List<ActivityInstance> list = runtimeService.createActivityInstanceQuery()
                .processInstanceId(instanceId)
                .list();
        return list.stream()
                .map(e -> {
                    FlowActivity flowActivity = new FlowActivity(e);
                    flowActivity.setMulti(ApprovalMultiEnum.SEQUENTIAL);
                    if ("userTask".equals(flowActivity.getActivityType())) {
                        Optional<FlowTask> optional = flowTasks.stream().filter(t -> t.getId().equals(e.getTaskId())).findFirst();
                        optional.ifPresent(flowTask -> flowActivity.setStatus(flowTask.getStatus()));
                        String assignee = flowActivity.getAssignee();
                        if (Objects.isNull(assignee)) {
                            // flowActivity.setAssignee(processInstance.getStartUserId());
                            flowActivity.setAssignee(userService.getByUsername(processInstance.getStartUserId()).getName());
                        }
                        ApprovalNode approvalNode = flowDefine.findFlowNodeOfType(ApprovalNode.class, e.getActivityId());
                        if (Objects.nonNull(approvalNode)) {
                            ApprovalMultiEnum multi = approvalNode.getMulti();
                            flowActivity.setMulti(multi);
                        }
                    } else if ("copyServiceTask".equals(flowActivity.getActivityType())) {
                        flowActivity.setMulti(ApprovalMultiEnum.JOINT);
                    } else if ("startEvent".equals(flowActivity.getActivityType())) {
                        // flowActivity.setAssignee(processInstance.getStartUserId());
                        flowActivity.setAssignee(userService.getByUsername(processInstance.getStartUserId()).getName());
                    } else if ("jumpServiceTask".equals(flowActivity.getActivityType())) {
                        flowActivity.setAssignee("admin");
                        if (StringUtils.isBlank(flowActivity.getDeleteReason())) {
                            flowActivity.setDeleteReason("<span>跳转失败 <span class='color-error'>不满足跳转条件</span></span>");
                        }
                    }
                    return flowActivity;
                })
                .sorted(Comparator.comparing(FlowActivity::getId))
                .collect(Collectors.toList());
    }

    @Override
    public List<FlowActivity> historyRecords(String instanceId) {
        HistoricProcessInstance historicProcessInstance = historyService.createHistoricProcessInstanceQuery()
                .processInstanceId(instanceId)
                .singleResult();
        FlowDefine flowDefine = flowDefineService.getDefine(historicProcessInstance.getProcessDefinitionId());
        List<FlowTask> flowTasks = flowTaskService.lambdaQuery()
                .select(FlowTask::getId, FlowTask::getStatus)
                .eq(FlowTask::getInstanceId, instanceId)
                .list();
        List<HistoricActivityInstance> list = historyService.createHistoricActivityInstanceQuery()
                .processInstanceId(instanceId)
                .list();
        return list.stream()
                .map(e -> {
                    FlowActivity flowActivity = new FlowActivity(e);
                    flowActivity.setMulti(ApprovalMultiEnum.SEQUENTIAL);
                    if ("userTask".equals(flowActivity.getActivityType())) {
                        Optional<FlowTask> optional = flowTasks.stream().filter(t -> t.getId().equals(e.getTaskId())).findFirst();
                        optional.ifPresent(flowTask -> flowActivity.setStatus(flowTask.getStatus()));
                        String assignee = flowActivity.getAssignee();
                        if (Objects.isNull(assignee)) {
                            flowActivity.setAssignee(historicProcessInstance.getStartUserId());
                        }
                        ApprovalNode approvalNode = flowDefine.findFlowNodeOfType(ApprovalNode.class, e.getActivityId());
                        if (Objects.nonNull(approvalNode)) {
                            ApprovalMultiEnum multi = approvalNode.getMulti();
                            flowActivity.setMulti(multi);
                        }
                    } else if ("copyServiceTask".equals(flowActivity.getActivityType())) {
                        flowActivity.setMulti(ApprovalMultiEnum.JOINT);
                    } else if ("startEvent".equals(flowActivity.getActivityType())) {
                        flowActivity.setAssignee(historicProcessInstance.getStartUserId());
                    } else if ("jumpServiceTask".equals(flowActivity.getActivityType())) {
                        flowActivity.setAssignee("admin");
                        if (StringUtils.isBlank(flowActivity.getDeleteReason())) {
                            flowActivity.setDeleteReason("<span>跳转失败 <span class='color-error'>不满足跳转条件</span></span>");
                        }
                    }
                    return flowActivity;
                })
                .sorted(Comparator.comparing(FlowActivity::getId))
                .collect(Collectors.toList());
    }
}
