package com.flow.service.impl;

import com.flow.common.mybatis.constant.SqlConstant;
import com.flow.common.mybatis.service.impl.BaseServiceImpl;
import com.flow.dao.FlowGroupDao;
import com.flow.entity.FlowGroup;
import com.flow.service.FlowGroupService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;
import java.util.Set;


@Service
public class FlowGroupServiceImpl extends BaseServiceImpl<FlowGroupDao, FlowGroup> implements FlowGroupService {
    @Autowired
    private FlowGroupDao flowGroupDao;

    @Override
    public List<FlowGroup> list(FlowGroup flowGroup) {
        return flowGroupDao.lambdaQueryChain()
                .setEntity(flowGroup)
                .orderByAsc(FlowGroup::getSort)
                .list();
    }

    @Override
    public FlowGroup create(FlowGroup flowGroup) {
        if (StringUtils.isBlank(flowGroup.getName())) {
            throw new RuntimeException("分组名称不能为空");
        }
        Optional<FlowGroup> optional = flowGroupDao.lambdaQueryChain()
                .select(FlowGroup::getSort)
                .orderByDesc(FlowGroup::getSort)
                .last(SqlConstant.LIMIT1)
                .oneOpt();
        if (optional.isPresent()) {
            FlowGroup group = optional.get();
            flowGroup.setSort(group.getSort() + 1);
        }
        flowGroupDao.insert(flowGroup);
        return flowGroup;
    }

    @Override
    public FlowGroup update(FlowGroup flowGroup) {
        if (StringUtils.isBlank(flowGroup.getName())) {
            throw new RuntimeException("分组名称不能为空");
        }
        flowGroupDao.updateById(flowGroup);
        return flowGroup;
    }

    @Override
    public void delete(Set<Long> ids) {
        flowGroupDao.deleteBatchIds(ids);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void sort(String newId, String oldId) {
        FlowGroup newGroup = flowGroupDao.selectById(newId);
        FlowGroup oldGroup = flowGroupDao.selectById(oldId);
        Integer newSort = newGroup.getSort();
        Integer oldSort = oldGroup.getSort();
        newGroup.setSort(oldSort);
        oldGroup.setSort(newSort);
        flowGroupDao.updateById(newGroup);
        flowGroupDao.updateById(oldGroup);
    }
}
