package com.flow.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.flow.common.core.exception.BaseException;
import com.flow.common.core.model.PageResult;
import com.flow.common.core.util.SecurityContextUtil;
import com.flow.common.mybatis.service.impl.BaseServiceImpl;
import com.flow.common.redis.service.RedisService;
import com.flow.dao.FlowTaskDao;
import com.flow.entity.FlowDefine;
import com.flow.entity.FlowInstance;
import com.flow.entity.FlowTask;
import com.flow.entity.node.ApprovalNode;
import com.flow.entity.node.FieldProperty;
import com.flow.entity.node.Node;
import com.flow.entity.node.RootNode;
import com.flow.enums.ProcessStatus;
import com.flow.flowable.cmd.*;
import com.flow.flowable.utils.ProcessElementUtil;
import com.flow.model.*;
import com.flow.service.FlowDefineService;
import com.flow.service.FlowInstanceService;
import com.flow.service.FlowTaskService;
import org.flowable.bpmn.model.BpmnModel;
import org.flowable.bpmn.model.UserTask;
import org.flowable.common.engine.impl.identity.Authentication;
import org.flowable.engine.*;
import org.flowable.engine.runtime.ActivityInstance;
import org.flowable.engine.task.Attachment;
import org.flowable.engine.task.Comment;
import org.flowable.task.api.DelegationState;
import org.flowable.task.api.Task;
import org.flowable.task.api.TaskInfo;
import org.flowable.task.api.history.HistoricTaskInstance;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class FlowTaskServiceImpl extends BaseServiceImpl<FlowTaskDao, FlowTask> implements FlowTaskService {
    @Autowired
    private FlowTaskDao flowTaskDao;
    @Autowired
    private TaskService taskService;
    @Autowired
    private RuntimeService runtimeService;
    @Autowired
    private HistoryService historyService;
    @Autowired
    private FlowDefineService flowDefineService;
    @Autowired
    private FlowInstanceService flowInstanceService;
    @Autowired
    private ManagementService managementService;
    @Autowired
    private RepositoryService repositoryService;
    @Autowired
    private RedisService<String> redisService;

    @Override
    public PageResult<FlowTask> todo(FlowTaskQuery flowTaskQuery) {
        flowTaskQuery.setAssignee(SecurityContextUtil.getUserId());
        Page<FlowInstance> page = new Page<>(
                flowTaskQuery.getPage(),
                flowTaskQuery.getLimit()
        );
        IPage<FlowTask> iPage = flowTaskDao.todo(page, flowTaskQuery);
        return new PageResult<>(iPage.getTotal(), iPage.getRecords());
    }

    @Override
    public PageResult<FlowTask> done(FlowTaskQuery flowTaskQuery) {
        flowTaskQuery.setAssignee(SecurityContextUtil.getUserId());
        Page<FlowInstance> page = new Page<>(
                flowTaskQuery.getPage(),
                flowTaskQuery.getLimit()
        );
        IPage<FlowTask> iPage = flowTaskDao.done(page, flowTaskQuery);
        return new PageResult<>(iPage.getTotal(), iPage.getRecords());
    }

    @Override
    public FormInfo getForm(String taskId) {
        TaskInfo task = this.getTask(taskId);
        if (Objects.isNull(task)) {
            throw new BaseException("任务不存在");
        }
        FlowDefine flowDefine = flowDefineService.getDefine(task.getProcessDefinitionId());
        if (Objects.isNull(flowDefine)) {
            throw new BaseException("流程不存在");
        }
        Map<String, Object> variables = this.getVariables(task.getId());
        ApprovalNode approvalNode = flowDefine.getNode(task.getTaskDefinitionKey(), ApprovalNode.class);
        RootNode rootNode = flowDefine.getNode(null, RootNode.class);
        List<FieldProperty> formProperties = Objects.nonNull(approvalNode) ? approvalNode.getFormProperties() : rootNode.getFormProperties();
        HashMap<String, Boolean> startOpera = new HashMap<>();
        startOpera.put("complete", true);
        Map<String, Boolean> operations = Objects.nonNull(approvalNode) ? approvalNode.getOperations() : startOpera;
        FormInfo formInfo = new FormInfo(
                flowDefine,
                formProperties,
                variables,
                operations
        );
        FlowInstance instance = flowInstanceService.getInstance(task.getProcessInstanceId());
        formInfo.setInstanceId(task.getProcessInstanceId());
        formInfo.setTaskId(task.getId());
        formInfo.setFormName(instance.getName());
        formInfo.setStartUserId(instance.getStartUserId());
        formInfo.setCancelDays(instance.getCancelDays());
        formInfo.setIsCancel(instance.getIsCancel());
        formInfo.setStartTime(instance.getStartTime());
        if (task instanceof HistoricTaskInstance) {
            HistoricTaskInstance historicTaskInstance = (HistoricTaskInstance) task;
            Date endTime = historicTaskInstance.getEndTime();
            if (Objects.nonNull(endTime)) {
                formInfo.setEndTime(
                        LocalDateTime.ofInstant(
                                endTime.toInstant(),
                                TimeZone.getDefault().toZoneId()
                        )
                );
            }
        }
        formInfo.setStatus(instance.getStatus());
        return formInfo;
    }

    @Override
    public TaskInfo getTask(String taskId) {
        Task task = taskService.createTaskQuery().taskId(taskId).singleResult();
        if (Objects.nonNull(task)) {
            return task;
        }
        return historyService.createHistoricTaskInstanceQuery().taskId(taskId).singleResult();
    }

    @Override
    public Map<String, Object> getVariables(String taskId) {
        TaskInfo task = this.getTask(taskId);
        if (Objects.isNull(task)) {
            throw new BaseException("任务不存在");
        }
        Map<String, Object> variables = new HashMap<>();
        if (task instanceof HistoricTaskInstance) {
            historyService.createHistoricVariableInstanceQuery().taskId(taskId).list().forEach(variableInstance ->
                    variables.putIfAbsent(variableInstance.getVariableName(), variableInstance.getValue())
            );
            historyService.createHistoricVariableInstanceQuery().executionId(task.getProcessInstanceId()).list().forEach(variableInstance ->
                    variables.putIfAbsent(variableInstance.getVariableName(), variableInstance.getValue())
            );
        } else {
            taskService.getVariableInstances(taskId).values().forEach(variableInstance ->
                    variables.putIfAbsent(variableInstance.getName(), variableInstance.getValue())
            );
            runtimeService.getVariableInstances(task.getProcessInstanceId()).values().forEach(variableInstance ->
                    variables.putIfAbsent(variableInstance.getName(), variableInstance.getValue())
            );
        }
        return variables;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveForm(TaskForm taskForm) {
        taskService.setVariables(taskForm.getTaskId(), taskForm.getValues());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void reject(RejectTaskForm taskForm) {
        Task task = this.comment(taskForm);
        this.updateStatus(task.getId(), ProcessStatus.REJECT);
        FlowDefine flowDefine = flowDefineService.getDefine(task.getProcessDefinitionId());
        if (Objects.nonNull(flowDefine)) {
            Node node = flowDefine.findFlowNodeOfType(Node.class, taskForm.getTargetNode());
            if (Objects.nonNull(node)) {
                taskService.addComment(
                        task.getId(),
                        task.getProcessInstanceId(),
                        String.format("<span>退回至 <span class='color-error'>%s</span></span>", node.getName())
                );
            }
        }
        // 回退节点
        try {
            redisService.del(task.getProcessInstanceId());
            managementService.executeCommand(new JumpActivityCmd(
                    task.getProcessInstanceId(),
                    task.getTaskDefinitionKey(),
                    taskForm.getTargetNode()
            ));
        } catch (Exception e) {
            throw new BaseException(e.getMessage());
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void retract(TaskForm taskForm) {
        managementService.executeCommand(new TaskRecallCmd(taskForm.getTaskId()));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void refuse(TaskForm taskForm) {
        Task task = this.comment(taskForm);
        this.updateStatus(task.getId(), ProcessStatus.REFUSE);
        flowInstanceService.reject(task.getProcessInstanceId());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void autoRefuse(String taskId) {
        this.autoRefuse(taskId, null);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void autoRefuse(String taskId, String content) {
        TaskInfo task = this.getTask(taskId);
        if (Objects.isNull(task)) {
            throw new BaseException("任务不存在");
        }
        TaskForm taskForm = new TaskForm();
        taskForm.setTaskId(task.getId());
        FlowComment flowComment = new FlowComment();
        flowComment.setUserId("admin");
        flowComment.setTime(new Date());
        flowComment.setContent(String.format("<span><span class='color-error'>自动拒绝</span>%s</span>", Optional.ofNullable(content).orElse("")));
        taskForm.setComment(flowComment);
        this.refuse(taskForm);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void transfer(TransferTaskForm taskForm) {
        Task task = this.comment(taskForm);
        taskService.setAssignee(task.getId(), taskForm.getAssignee());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addSign(AddSignTaskForm taskForm) {
        Task taskInfo = this.comment(taskForm);
        try {
            managementService.executeCommand(new AddTaskExecutionCmd(
                    taskInfo.getProcessInstanceId(),
                    taskInfo.getTaskDefinitionKey(),
                    taskForm.getAssignee())
            );
        } catch (Exception e) {
            throw new BaseException(e.getMessage());
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void removeSign(AddSignTaskForm taskForm) {
        Task taskInfo = this.comment(taskForm);
        try {
            managementService.executeCommand(new DeleteTaskExecutionCmd(
                    taskInfo,
                    taskForm.getAssignee()
            ));
        } catch (Exception e) {
            throw new BaseException(e.getMessage());
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delegate(TransferTaskForm taskForm) {
        Task task = this.comment(taskForm);
        taskService.delegateTask(task.getId(), taskForm.getAssignee());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateStatus(String id, ProcessStatus status) {
        this.updateStatus(id, status, null);
    }

    @Override
    public void updateStatus(String id, ProcessStatus status, Map<String, Object> variables) {
        FlowTask flowTask = new FlowTask();
        flowTask.setId(id);
        flowTask.setStatus(status);
        flowTask.setVariables(variables);
        this.updateById(flowTask);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void complete(TaskForm taskForm) {
        Task task = this.comment(taskForm);
        Map<String, Object> localVariables = new HashMap<>();
        Map<String, Object> variables = runtimeService.getVariables(task.getProcessInstanceId());
        Map<String, Object> values = Optional.ofNullable(taskForm.getValues()).orElse(new HashMap<>());
        values.forEach((key, value) -> {
            if (variables.containsKey(key)) {
                Object obj = variables.get(key);
                if (Objects.nonNull(obj) && !obj.equals(value)) {
                    localVariables.put(key, value);
                }
            }
        });
        String assignee = SecurityContextUtil.getUserId();
        if (StringUtils.isNotBlank(assignee)) {
            redisService.hset(task.getProcessInstanceId(), assignee, task.getTaskDefinitionKey());
        }
        DelegationState delegationState = task.getDelegationState();
        if (Objects.nonNull(delegationState) && delegationState == DelegationState.PENDING) {
            taskService.resolveTask(
                    taskForm.getTaskId(),
                    values
            );
        } else {
            taskService.setVariablesLocal(taskForm.getTaskId(), localVariables);
            taskService.complete(
                    taskForm.getTaskId(),
                    values
            );
            this.updateStatus(
                    taskForm.getTaskId(),
                    ProcessStatus.COMPLETED,
                    localVariables
            );
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void copy(CopyTaskForm taskForm) {
        Task task = this.comment(taskForm);
        List<String> copyTo = taskForm.getCopyTo();
        for (String to : copyTo) {
            managementService.executeCommand(new CopyProcessInstanceCmd(task.getProcessInstanceId(), task.getId(), to));
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void autoComplete(String taskId) {
        this.autoComplete(taskId, null);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void autoComplete(String taskId, String content) {
        TaskInfo task = this.getTask(taskId);
        if (Objects.isNull(task)) {
            throw new BaseException("任务不存在");
        }
        Map<String, Object> variables = new HashMap<>();
        TaskForm taskForm = new TaskForm();
        taskForm.setTaskId(task.getId());
        taskForm.setValues(variables);
        FlowComment flowComment = new FlowComment();
        flowComment.setUserId("admin");
        flowComment.setTime(new Date());
        flowComment.setContent(String.format("<span><span class='color-primary'>自动通过</span>%s</span>", Optional.ofNullable(content).orElse("")));
        taskForm.setComment(flowComment);
        this.complete(taskForm);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Task comment(TaskForm taskForm) {
        Task task = taskService.createTaskQuery().taskId(taskForm.getTaskId()).singleResult();
        if (Objects.isNull(task)) {
            throw new BaseException("任务不存在");
        }
        if (task.isSuspended()) {
            throw new BaseException("任务已挂起");
        }
        Authentication.setAuthenticatedUserId(SecurityContextUtil.getUserId());
        FlowComment flowComment = taskForm.getComment();
        List<FlowAttachment> attachments = flowComment.getAttachments();
        String content = flowComment.getContent();
        if (CollectionUtils.isEmpty(attachments) && StringUtils.isBlank(content)) {
            return task;
        }
        // 评论
        Comment comment = taskService.addComment(
                task.getId(),
                task.getProcessInstanceId(),
                flowComment.getContent()
        );
        // 附件
        List<FlowAttachment> flowAttachments = flowComment.getAttachments();
        if (CollectionUtils.isNotEmpty(flowAttachments)) {
            for (FlowAttachment flowAttachment : flowAttachments) {
                taskService.createAttachment(
                        flowAttachment.getType(),
                        task.getId(),
                        task.getProcessInstanceId(),
                        flowAttachment.getName(),
                        comment.getId(),
                        flowAttachment.getUrl()
                );
            }
        }
        return task;
    }

    @Override
    public List<FlowComment> getComments(String taskId) {
        List<Comment> taskComments = taskService.getTaskComments(taskId);
        List<Attachment> taskAttachments = taskService.getTaskAttachments(taskId);
        return taskComments.stream()
                .map(comment -> {
                    List<Attachment> attachments = taskAttachments.stream()
                            .filter(a -> comment.getId().equals(a.getDescription()))
                            .collect(Collectors.toList());
                    return new FlowComment(comment, attachments);
                })
                .sorted(Comparator.comparing(FlowComment::getTime))
                .collect(Collectors.toList());
    }

    @Override
    public List<Node> getParentNodes(String taskId) {
        TaskInfo taskInfo = this.getTask(taskId);
        if (Objects.isNull(taskInfo)) {
            throw new BaseException("任务不存在");
        }
        BpmnModel model = repositoryService.getBpmnModel(taskInfo.getProcessDefinitionId());
        Set<UserTask> parentElements = ProcessElementUtil.findParentElements(model.getFlowElement(taskInfo.getTaskDefinitionKey()), UserTask.class);
        String sql = "SELECT t1.ID_,t1.ACT_ID_,t1.ACT_NAME_ FROM ACT_RU_ACTINST AS t1 WHERE " +
                "END_TIME_ = ( SELECT MAX( END_TIME_ ) FROM ACT_RU_ACTINST AS t2 WHERE t1.PROC_INST_ID_ = t2.PROC_INST_ID_ AND t1.ACT_ID_ = t2.ACT_ID_ ) " +
                "AND t1.PROC_INST_ID_ = #{processInstanceId} " +
                "AND t1.ACT_TYPE_ = 'userTask' " +
                "AND t1.END_TIME_ IS NOT NULL " +
                "ORDER BY t1.END_TIME_ DESC";
        List<ActivityInstance> activityInstances = runtimeService.createNativeActivityInstanceQuery()
                .sql(sql)
                .parameter("processInstanceId", taskInfo.getProcessInstanceId())
                .list();
        return activityInstances.stream()
                .filter(activityInstance -> parentElements.stream().anyMatch(userTask -> userTask.getId().equals(activityInstance.getActivityId())))
                .map(activity -> {
                    Node node = new ApprovalNode();
                    node.setId(activity.getActivityId());
                    node.setName(activity.getActivityName());
                    return node;
                })
                .collect(Collectors.toList());
    }

    @Override
    public Set<String> getMultiInstanceTaskUnUsers(String taskId) {
        Task task = taskService.createTaskQuery()
                .taskId(taskId)
                .singleResult();
        if (Objects.isNull(task)) {
            throw new BaseException("任务不存在");
        }
        Set<String> unUsers = new HashSet<>();
        List<ActivityInstance> activityList = runtimeService.createActivityInstanceQuery()
                .processInstanceId(task.getProcessInstanceId())
                .activityId(task.getTaskDefinitionKey())
                .list();
        for (ActivityInstance activityInstance : activityList) {
            if (Objects.isNull(activityInstance.getEndTime())) {
                unUsers.add(activityInstance.getAssignee());
            }
        }
        String collectionName = String.format("%sCollection", task.getTaskDefinitionKey());
        List<String> assigneeList = runtimeService.getVariable(task.getExecutionId(), collectionName, List.class);
        for (String assignee : assigneeList) {
            if (!unUsers.contains(assignee)) {
                boolean noneMatch = activityList
                        .stream()
                        .noneMatch(activityInstance -> activityInstance.getAssignee().equals(assignee));
                if (noneMatch) {
                    unUsers.add(assignee);
                }
            }
        }
        return unUsers;
    }
}
