package com.flow.controller;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.flow.common.core.model.Result;
import com.flow.entity.DatasetField;
import com.flow.entity.FlowDefine;
import com.flow.entity.FlowModel;
import com.flow.entity.FlowModelHistory;
import com.flow.model.spreadsheet.SpreadSheet;
import com.flow.service.FlowModelService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.List;
import java.util.Set;

@RestController
@RequestMapping("/model")
public class FlowModelController {
    @Autowired
    private FlowModelService flowModelService;

    @GetMapping
    public Result<List<FlowModel>> list(FlowModel flowModel) {
        return Result.success(flowModelService.list(flowModel));
    }

    @PostMapping
    public Result<FlowModel> create(@RequestBody FlowModel flowModel) {
        return Result.success(flowModelService.create(flowModel));
    }

    @PutMapping
    public Result<FlowModel> update(@RequestBody FlowModel flowModel) {
        return Result.success(flowModelService.update(flowModel));
    }

    @PutMapping("/suspend/{modelId}")
    public Result<?> suspend(@PathVariable Long modelId) {
        return Result.success(flowModelService.suspend(modelId));
    }

    @PreAuthorize("hasAnyAuthority('flow:model:del')")
    @DeleteMapping
    public Result<?> delete(@RequestBody Set<Long> ids) {
        flowModelService.delete(ids);
        return Result.success();
    }

    @CrossOrigin(origins = "*", maxAge = 3600)
    @PostMapping("/download")
    public void downloadXml(@RequestBody FlowModel flowModel) throws IOException {
        flowModelService.downloadXml(flowModel);
    }

    @PostMapping("/deploy/{modelId}")
    public Result<FlowDefine> deploy(@PathVariable Long modelId) throws JsonProcessingException {
        return Result.success(flowModelService.deploy(modelId));
    }

    @GetMapping("/{modelCode}")
    public Result<FlowModel> getModel(@PathVariable String modelCode) {
        FlowModel model = flowModelService.getModel(modelCode);
        return Result.success(model);
    }

    @GetMapping("/history/{modelCode}")
    public Result<List<FlowModelHistory>> getModelHistory(@PathVariable String modelCode) {
        List<FlowModelHistory> modelHistory = flowModelService.getModelHistory(modelCode);
        return Result.success(modelHistory);
    }

    @GetMapping("/switch/{historyModelId}")
    public Result<?> switchModel(@PathVariable Long historyModelId) {
        flowModelService.switchModel(historyModelId);
        return Result.success();
    }

    @GetMapping("/report/{modelId}")
    public Result<List<DatasetField>> getReportField(@PathVariable String modelId) {
        List<DatasetField> list = flowModelService.getReportField(modelId);
        return Result.success(list);
    }

    @PostMapping("/report/save/{modelId}")
    public Result<?> savePrint(@PathVariable String modelId,@RequestBody SpreadSheet spreadSheet) {
        return Result.success(flowModelService.savePrint(spreadSheet,modelId));
    }

    @GetMapping("/report/print/{modelId}")
    public Result<SpreadSheet> getPrint(@PathVariable String modelId) {
        SpreadSheet spreadSheet = flowModelService.getPrint(modelId);
        return Result.success(spreadSheet);
    }

    @PostMapping("/report/excel/{modelId}")
    public void exportExcel(@PathVariable String modelId) throws IOException {
        flowModelService.exportExcel(modelId);
    }
}