package com.flow.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.flow.common.mybatis.entity.BaseEntity;
import com.flow.entity.node.BranchNode;
import com.flow.entity.node.ConditionNode;
import com.flow.entity.node.Node;
import com.flow.entity.settings.Settings;
import com.flow.model.ModelIcon;
import com.google.common.collect.Sets;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.flowable.engine.repository.ProcessDefinition;

import java.time.LocalDateTime;
import java.util.*;

@EqualsAndHashCode(callSuper = true)
@Data
@AllArgsConstructor
@NoArgsConstructor
@TableName(value = "flow_define", resultMap = "BaseResultMap")
public class FlowDefine extends BaseEntity {
    private String defineId;
    @JsonSerialize(using = ToStringSerializer.class)
    private Long modelId;
    @JsonSerialize(using = ToStringSerializer.class)
    private Long groupId;
    @TableField(exist = false)
    private String key;
    @TableField(exist = false)
    private String name;
    @TableField(exist = false)
    private Integer version;
    @TableField(value = "_admin", typeHandler = JacksonTypeHandler.class)
    private List<String> admin;
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<Principal> starters;
    @TableField(typeHandler = JacksonTypeHandler.class)
    private ModelIcon icon;
    @TableField(typeHandler = JacksonTypeHandler.class)
    private Node process;
    @TableField(typeHandler = JacksonTypeHandler.class)
    private FormModel form;
    @TableField(typeHandler = JacksonTypeHandler.class)
    private Settings settings;
    @TableField(exist = false)
    private String deploymentId;
    @TableField(exist = false)
    private String description;
    @TableField(exist = false)
    private Integer suspend;

    public FlowDefine(ProcessDefinition processDefinition) {
        this.defineId = processDefinition.getId();
        this.key = processDefinition.getKey();
        this.name = processDefinition.getName();
        this.version = processDefinition.getVersion();
        this.deploymentId = processDefinition.getDeploymentId();
        this.description = processDefinition.getDescription();
        this.suspend = processDefinition.isSuspended() ? 2 : 1;
    }


    public <T extends BranchNode> Set<T> numberOfNestedBranchNodes(String nodeId, Class<T> clazz) {
        return numberOfNestedBranchNodes(nodeId, null, clazz);
    }

    private <T extends BranchNode> Set<T> numberOfNestedBranchNodes(String nodeId, String childId, Class<T> clazz) {
        Set<T> list = Sets.newHashSet();
        Node node = allNodesAsMap().get(nodeId);
        if (Objects.nonNull(node)) {
            if (clazz.isInstance(node)) {
                BranchNode branchNode = (BranchNode) node;
                List<ConditionNode> branches = branchNode.getBranches();
                if (branches.stream().anyMatch(c -> c.getId().equals(childId))) {
                    list.add(clazz.cast(node));
                }
            }
            if (StringUtils.isNotBlank(node.getPid())) {
                Set<T> branches = numberOfNestedBranchNodes(node.getPid(), node.getId(), clazz);
                if (CollectionUtils.isNotEmpty(branches)) {
                    list.addAll(branches);
                }
            }
        }
        return list;
    }

    public <T extends Node> List<T> flatNodesOfType(Node node, Class<T> clazz) {
        List<T> foundFlowNodes = new ArrayList<>();
        if (clazz.isInstance(node)) {
            foundFlowNodes.add(clazz.cast(node));
        }
        if (node instanceof BranchNode) {
            BranchNode branchNode = (BranchNode) node;
            for (Node branch : branchNode.getBranches()) {
                foundFlowNodes.addAll(flatNodesOfType(branch, clazz));
            }
        }
        Node next = node.getNext();
        if (next != null) {
            foundFlowNodes.addAll(flatNodesOfType(next, clazz));
        }
        return foundFlowNodes;
    }

    public <T extends Node> List<T> findFlowNodesOfType(Class<T> clazz) {
        return this.findFlowNodesOfType(clazz, process);
    }

    public <T extends Node> List<T> findFlowNodesOfType(Class<T> clazz, Node node) {
        List<T> foundFlowNodes = new ArrayList<>();
        if (clazz.isInstance(node)) {
            foundFlowNodes.add(clazz.cast(node));
        }
        if (node instanceof BranchNode) {
            BranchNode branchNode = (BranchNode) node;
            for (Node branch : branchNode.getBranches()) {
                foundFlowNodes.addAll(findFlowNodesOfType(clazz, branch));
            }
        }
        Node next = node.getNext();
        if (next != null) {
            foundFlowNodes.addAll(findFlowNodesOfType(clazz, next));
        }
        return foundFlowNodes;
    }

    public <T extends Node> T findFlowNodeOfType(Class<T> clazz, String nodeId) {
        return findFlowNodeOfType(clazz, nodeId, process);
    }

    private <T extends Node> T findFlowNodeOfType(Class<T> clazz, String nodeId, Node node) {
        if (nodeId.equals(node.getId())) {
            if (clazz.isInstance(node)) {
                return clazz.cast(node);
            }
            return null;
        }
        if (node instanceof BranchNode) {
            BranchNode branchNode = (BranchNode) node;
            for (Node branch : branchNode.getBranches()) {
                T result = findFlowNodeOfType(clazz, nodeId, branch);
                if (Objects.nonNull(result)) return result;
            }
        }
        Node next = node.getNext();
        if (next != null) {
            T result = findFlowNodeOfType(clazz, nodeId, next);
            if (Objects.nonNull(result)) return result;
        }
        return null;
    }

    public <T extends Node> T getNode(String nodeId, Class<T> type) {
        Map<String, Node> nodeMap = allNodesAsMap();
        if (StringUtils.isNotBlank(nodeId)) {
            Node node = nodeMap.get(nodeId);
            if (Objects.nonNull(node)) {
                if (type.isInstance(node)) {
                    return type.cast(node);
                } else {
                    return null;
                }
            }
        } else {
            for (Node value : nodeMap.values()) {
                if (type.isInstance(value)) {
                    return type.cast(value);
                }
            }
        }
        return null;
    }

    public Map<String, Node> allNodesAsMap() {
        Map<String, Node> result = new HashMap<>();
        List<Node> allNodes = listAllNodes();
        if (allNodes != null) {
            for (Node node : allNodes) {
                if (!result.containsKey(node.getId())) {
                    result.put(node.getId(), node);
                }
            }
        }
        return result;
    }

    public List<Node> listAllNodes() {
        List<Node> listOfAllNodes = new ArrayList<>();
        collectSubNodes(process, listOfAllNodes);
        return listOfAllNodes;
    }

    public void collectSubNodes(Node node, List<Node> listOfAllNodes) {
        if (Objects.nonNull(node)) {
            listOfAllNodes.add(node);
            if (node instanceof BranchNode) {
                BranchNode branchNode = (BranchNode) node;
                for (Node branch : branchNode.getBranches()) {
                    collectSubNodes(branch, listOfAllNodes);
                }
            }
            collectSubNodes(node.getNext(), listOfAllNodes);
        }
    }

}
