package com.flow.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.flow.enums.ProcessStatus;
import lombok.Data;

import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.List;
import java.util.Objects;
@JsonInclude(JsonInclude.Include.ALWAYS)
@TableName(value = "flow_instance", autoResultMap = true)
@Data
public class FlowInstance {
    private String id;
    private ProcessStatus status;
    private Integer cancelDays;
    @TableField(exist = false)
    private Boolean isCancel = false;
    @TableField(exist = false)
    private String name;
    @TableField(exist = false)
    private String definitionId;
    @TableField(exist = false)
    private Integer version;
    @TableField(exist = false)
    private String startUserId;
    @TableField(exist = false)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime startTime;
    @TableField(exist = false)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime endTime;
    @TableField(exist = false)
    private Long duration;
    @TableField(typeHandler = JacksonTypeHandler.class, exist = false)
    private List<String> currentNodes;

    public static FlowInstance buildStatus(String id, ProcessStatus status) {
        FlowInstance flowInstance = new FlowInstance();
        flowInstance.setId(id);
        flowInstance.setStatus(status);
        return flowInstance;
    }

    public void setStartTime(LocalDateTime startTime) {
        this.startTime = startTime;
        long milliSecond = LocalDateTime.now().toInstant(ZoneOffset.UTC).toEpochMilli();
        this.duration = milliSecond - startTime.toInstant(ZoneOffset.UTC).toEpochMilli();
    }

    public void setCancelDays(Integer cancelDays) {
        this.cancelDays = cancelDays;
        if (this.status == ProcessStatus.RUNNING) {
            if (Objects.nonNull(this.startTime)) {
                LocalDateTime exp = this.startTime.plusDays(cancelDays);
                if (LocalDateTime.now().isBefore(exp)) {
                    this.isCancel = true;
                }
            }
        }
    }
}
