package com.flow.entity;

import com.baomidou.mybatisplus.annotation.SqlCondition;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.flow.common.mybatis.entity.BaseEntity;
import com.flow.entity.node.Node;
import com.flow.entity.settings.Settings;
import com.flow.model.ModelIcon;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.flowable.bpmn.BpmnAutoLayout;
import org.flowable.bpmn.model.Process;
import org.flowable.bpmn.model.*;
import org.flowable.common.engine.api.delegate.event.FlowableEventType;

import java.util.List;
import java.util.stream.Collectors;

@EqualsAndHashCode(callSuper = true)
@Data
@TableName(value = "flow_model", autoResultMap = true)
public class FlowModel extends BaseEntity {
    private String code;
    @TableField(condition = SqlCondition.LIKE)
    private String name;
    @TableField(typeHandler = JacksonTypeHandler.class)
    private ModelIcon icon;
    @TableField(typeHandler = JacksonTypeHandler.class)
    private Node process;
    @TableField(typeHandler = JacksonTypeHandler.class)
    private FormModel form;
    private Integer version;
    private Integer sort;
    @JsonSerialize(using = ToStringSerializer.class)
    private Long groupId;
    private Boolean enable;
    private String remark;
    @TableField(value = "_admin", typeHandler = JacksonTypeHandler.class)
    private List<String> admin;
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<Principal> starters;
    @TableField(typeHandler = JacksonTypeHandler.class)
    private Settings settings;

    public BpmnModel toBpmnModel() {
        BpmnModel bpmnModel = new BpmnModel();
        // 命名空间
        bpmnModel.setTargetNamespace("https://flowable.org/bpmn20");
        // 创建一个流程
        Process process = new Process();
        // 设置流程的id
        process.setId(this.getCode());
        // 设置流程的name
        process.setName(this.getName());
        // 是否可执行
        process.setExecutable(this.getEnable());
        // 设置流程的文档
        process.setDocumentation(this.getRemark());
        // 递归构建所有节点
        Node node = this.getProcess();
        List<FlowElement> flowElementList = node.convert();
        for (FlowElement flowElement : flowElementList) {
            process.addFlowElement(flowElement);
        }
        // 流程管理员
        ExtensionElement adminElement = new ExtensionElement();
        adminElement.setNamespace("https://flowable.org/model");
        adminElement.setNamespacePrefix("flowable");
        adminElement.setName("admin");
        if (CollectionUtils.isNotEmpty(this.getAdmin())) {
            adminElement.setElementText(String.join(",", this.getAdmin()));
        }
        process.addExtensionElement(adminElement);
        // 设置谁可以发起流程
        if (CollectionUtils.isNotEmpty(this.starters)) {
            List<String> candidateStarterUsers = this.starters.stream()
                    .filter(principal -> principal.getType() == PrincipalType.USER)
                    .map(Principal::getId)
                    .collect(Collectors.toList());
            process.setCandidateStarterUsers(candidateStarterUsers);
            List<String> candidateStarterRoles = this.starters.stream()
                    .filter(principal -> principal.getType() == PrincipalType.ROLE)
                    .map(Principal::getId)
                    .collect(Collectors.toList());
            process.setCandidateStarterGroups(candidateStarterRoles);
        }
        // 设置流程
        bpmnModel.addProcess(process);
        // 自动布局
        new BpmnAutoLayout(bpmnModel).execute();
        return bpmnModel;
    }

    public EventListener buildEventListener(FlowableEventType eventType, String implementation) {
        EventListener eventListener = new EventListener();
        eventListener.setEvents(eventType.name());
        eventListener.setImplementationType(ImplementationType.IMPLEMENTATION_TYPE_DELEGATEEXPRESSION);
        eventListener.setImplementation(implementation);
        return eventListener;
    }
}
