package com.flow.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.flow.enums.ProcessStatus;
import lombok.Data;
import org.flowable.task.service.impl.persistence.entity.TaskEntityImpl;

import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.HashMap;
import java.util.Map;

@TableName(value = "flow_task", autoResultMap = true)
@Data
public class FlowTask {
    private String id;
    private ProcessStatus status;
    @TableField(exist = false)
    private String name;
    private String instanceId;
    @TableField(exist = false)
    private String instanceName;
    @TableField(exist = false)
    private String startUserId;
    @TableField(exist = false)
    private String definitionId;
    @TableField(exist = false)
    private String assignee;
    @TableField(exist = false)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime startTime;
    @TableField(exist = false)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime endTime;
    @TableField(exist = false)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime dueDate;
    @TableField(exist = false)
    private Long duration;
    @TableField(typeHandler = JacksonTypeHandler.class)
    private Map<String, Object> variables;

    public static FlowTask build(TaskEntityImpl entity) {
        FlowTask flowTask = new FlowTask();
        flowTask.setId(entity.getId());
        flowTask.setInstanceId(entity.getProcessInstanceId());
        flowTask.setVariables(new HashMap<>());
        flowTask.setStatus(ProcessStatus.RUNNING);
        return flowTask;
    }

    public void setStartTime(LocalDateTime startTime) {
        this.startTime = startTime;
        long milliSecond = LocalDateTime.now().toInstant(ZoneOffset.UTC).toEpochMilli();
        this.duration = milliSecond - startTime.toInstant(ZoneOffset.UTC).toEpochMilli();
    }
}
