package com.flow.entity;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.flow.entity.form.Container;
import com.flow.entity.form.Field;
import com.flow.entity.node.FieldProperty;
import lombok.Data;

import java.io.Serializable;
import java.util.*;
import java.util.stream.Collectors;

@Data
public class FormModel implements Serializable {
    private static final long serialVersionUID = 1L;
    private String labelPosition;
    private Integer labelWidth;
    private Integer gutter;
    private Boolean disabled;
    private Integer span;
    private List<Field> fields;
    private List<Object> rules;
    private List<Object> dataSource;

    public void initialization(List<FieldProperty> formProperties) {
        if (CollectionUtils.isNotEmpty(formProperties)) {
            Map<String, Field> fieldMap = this.allFieldsAsMap();
            for (FieldProperty fieldProperty : formProperties) {
                Field field = fieldMap.get(fieldProperty.getId());
                if (Objects.nonNull(field)) {
                    field.populateProperty(fieldProperty);
                }
            }
        }
    }

    public void initialization(Map<String, Object> variables) {
        if (CollectionUtils.isNotEmpty(variables)) {
            Map<String, Field> fieldMap = this.allFieldsAsMap();
            for (Map.Entry<String, Field> entry : fieldMap.entrySet()) {
                Field field = entry.getValue();
                if (Objects.nonNull(field)) {
                    field.populateValue(variables);
                }
            }
        }
    }

    public List<Field> findByName(String name) {
        List<Field> allFields = this.listAllFields();
        return allFields.stream()
                .filter(field -> name.equals(field.getName()))
                .collect(Collectors.toList());
    }

    public Map<String, Field> allFieldsAsMap() {
        Map<String, Field> result = new HashMap<>();
        List<Field> allFields = listAllFields();
        if (allFields != null) {
            for (Field field : allFields) {
                if (!result.containsKey(field.getId())) {
                    result.put(field.getId(), field);
                }
            }
        }
        return result;
    }

    public List<Field> listAllFields() {
        List<Field> listOfAllFields = new ArrayList<>();
        collectSubFields(fields, listOfAllFields);
        return listOfAllFields;
    }

    protected void collectSubFields(List<Field> fields, List<Field> listOfAllFields) {
        if (fields != null && !fields.isEmpty()) {
            for (Field field : fields) {
                listOfAllFields.add(field);
                if (field instanceof Container) {
                    Container container = (Container) field;
                    List<Field> subFields = container.getChildren();
                    if (subFields != null) {
                        collectSubFields(subFields, listOfAllFields);
                    }
                }
            }
        }
    }

}
