package com.flow.entity.form.serial;

import com.flow.common.core.util.ApplicationContextUtil;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.redisson.api.RAtomicLong;
import org.redisson.api.RedissonClient;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.concurrent.TimeUnit;

@EqualsAndHashCode(callSuper = true)
@Data
public class IncNumber extends SerialRule {
    private Integer digitsNum;
    private ResetEnum resetDuration;
    private Integer startValue;
    private Boolean fixedLength;
    private String chars;

    @Override
    public String generate(String key) {
        StringBuilder serialKey = new StringBuilder(key);
        serialKey.append(":");
        long timeToLive = 0;
        if (resetDuration == ResetEnum.DAY) {
            serialKey.append(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMdd")));
            timeToLive = 1;
        } else if (resetDuration == ResetEnum.WEEK) {
            serialKey.append(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyww")));
            timeToLive = 7;
        } else if (resetDuration == ResetEnum.MONTH) {
            serialKey.append(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMM")));
            timeToLive = 31;
        } else if (resetDuration == ResetEnum.YEAR) {
            serialKey.append(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy")));
            timeToLive = 365;
        }
        RedissonClient redissonClient = ApplicationContextUtil.getBean(RedissonClient.class);
        RAtomicLong atomicLong = redissonClient.getAtomicLong(serialKey.toString());
        if (!atomicLong.isExists()) {
            atomicLong.set(startValue);
            if (timeToLive > 0) {
                atomicLong.expire(timeToLive, TimeUnit.DAYS);
            }
        }
        return String.format("%" + digitsNum + "s", atomicLong.incrementAndGet()).replace(" ", "0");
    }
}
