package com.flow.entity.node;

import com.flow.flowable.model.CopyServiceTask;
import com.google.common.collect.Lists;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.flowable.bpmn.model.FieldExtension;
import org.flowable.bpmn.model.FlowElement;
import org.flowable.bpmn.model.MultiInstanceLoopCharacteristics;
import org.flowable.bpmn.model.SequenceFlow;

import java.util.*;

@EqualsAndHashCode(callSuper = true)
@Data
public class CcNode extends AssigneeNode {
    // 表单属性
    private List<FieldProperty> formProperties = new ArrayList<>();
    // 操作权限
    private Map<String, Boolean> operations = new LinkedHashMap<>();

    @Override
    public List<FlowElement> convert() {
        ArrayList<FlowElement> elements = Lists.newArrayList();
        // 服务节点
        CopyServiceTask copyServiceTask = new CopyServiceTask();
        copyServiceTask.setId(this.getId());
        copyServiceTask.setType("copy");
        copyServiceTask.setName(this.getName());
        copyServiceTask.setExecutionListeners(this.buidEventListener());
        // serviceTask.setAsynchronous(true);
        // copyServiceTask.setImplementationType(ImplementationType.IMPLEMENTATION_TYPE_DELEGATEEXPRESSION);
        // copyServiceTask.setImplementation("${ccBehavior}");
        String variable = String.format("%sItem", this.getId());
        MultiInstanceLoopCharacteristics multiInstanceLoopCharacteristics = new MultiInstanceLoopCharacteristics();
        multiInstanceLoopCharacteristics.setSequential(false);
        multiInstanceLoopCharacteristics.setElementVariable(variable);
        multiInstanceLoopCharacteristics.setInputDataItem(String.format("${%sCollection}", this.getId()));
        copyServiceTask.setLoopCharacteristics(multiInstanceLoopCharacteristics);
        String assignee = String.format("${%s}", variable);
        copyServiceTask.setAssignee(assignee);
        // 扩展属性
        FieldExtension fieldExtension = new FieldExtension();
        fieldExtension.setFieldName("assignee");
        fieldExtension.setExpression(assignee);
        copyServiceTask.getFieldExtensions().add(fieldExtension);
        elements.add(copyServiceTask);
        // 下一个节点的连线
        Node next = this.getNext();
        SequenceFlow sequenceFlow = this.buildSequence(next);
        elements.add(sequenceFlow);
        // 下一个节点
        if (Objects.nonNull(next)) {
            next.setBranchId(this.getBranchId());
            List<FlowElement> flowElements = next.convert();
            elements.addAll(flowElements);
        }
        return elements;
    }

}
