package com.flow.entity.node;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.flow.common.mybatis.entity.FilterCondition;
import com.flow.common.mybatis.entity.FilterGroup;
import com.google.common.collect.Lists;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.flowable.bpmn.model.FlowElement;
import org.flowable.bpmn.model.SequenceFlow;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

@EqualsAndHashCode(callSuper = true)
@Data
public class ConditionNode extends Node {
    private Boolean def;
    private FilterGroup conditions;
    @JsonIgnore
    private static Map<String, String> operatorMap = new HashMap<>();

    static {
        // 等于
        operatorMap.put("eq", "var:eq(%s, %s)");
        // 不等于
        operatorMap.put("ne", "var:notEquals(%s, %s)");
        // 包含
        operatorMap.put("in", "var:containsAny(%s, %s)");
        // 不包含
        operatorMap.put("ni", "!var:containsAny(%s, %s)");
        // 为空
        operatorMap.put("ul", "var:isNull(%s)");
        // 不为空
        operatorMap.put("nu", "var:isNotNull(%s)");
        // 字符包含
        operatorMap.put("lk", "var:contains(%s, %s)");
        // 字符不包含
        operatorMap.put("nl", "!var:contains(%s, %s)");
        // 大于
        operatorMap.put("gt", "var:gt(%s, %s)");
        // 小于
        operatorMap.put("lt", "var:lt(%s, %s)");
        // 小于或等于
        operatorMap.put("le", "var:lte(%s, %s)");
        // 大于或等于
        operatorMap.put("ge", "var:gte(%s, %s)");
        // 介于
        operatorMap.put("bt", "var:bt(%s, %s)");
        // 包含全部
        operatorMap.put("ia", "var:containsAll(%s, %s)");
        // 不包含全部
        operatorMap.put("na", "!var:containsAll(%s, %s)");
    }

    protected static String stringVal(Object val) {
        if (val instanceof String) {
            return String.format("'%s'", val);
        } else {
            return String.valueOf(val);
        }
    }

    public static String toExpression(FilterGroup filterGroup) {
        List<FilterCondition> conditions = filterGroup.getConditions();
        String expression = conditions.stream().map(condition -> {
            String operator = condition.getOperator();
            Object value = condition.getValue();
            if (StringUtils.isNotBlank(operator) && Objects.nonNull(value)) {
                if (value instanceof Collection) {
                    value = ((Collection<?>) value).stream()
                            .map(ConditionNode::stringVal)
                            .collect(Collectors.joining(","));
                    if ("in".equals(operator)) {
                        operator = "ia";
                    } else if ("ni".equals(operator)) {
                        operator = "na";
                    }
                } else if (value instanceof Object[]) {
                    value = Arrays.stream((Object[]) value)
                            .map(ConditionNode::stringVal)
                            .collect(Collectors.joining(","));
                    if ("in".equals(operator)) {
                        operator = "ia";
                    } else if ("ni".equals(operator)) {
                        operator = "na";
                    }
                } else {
                    value = stringVal(value);
                }
                String field = condition.getField();
                return String.format(operatorMap.get(operator), field, value);
            }
            return "";
        }).collect(Collectors.joining("and".equals(filterGroup.getOperator()) ? " && " : " ||"));
        if (CollectionUtils.isEmpty(filterGroup.getGroups())) {
            return expression;
        } else {
            String collect = filterGroup.getGroups()
                    .stream()
                    .map(ConditionNode::toExpression)
                    .collect(Collectors.joining("and".equals(filterGroup.getOperator()) ? " && " : " || "));
            if (StringUtils.isNotBlank(expression)) {
                return String.format("(%s) %s (%s)", expression, "and".equals(filterGroup.getOperator()) ? " && " : " || ", collect);
            } else {
                return collect;
            }
        }
    }

    @Override
    public List<FlowElement> convert() {
        ArrayList<FlowElement> elements = Lists.newArrayList();
        // 条件节点
        SequenceFlow sequenceFlow = this.buildSequence(this);
        sequenceFlow.setId(this.getId());
        sequenceFlow.setName(this.getName());
        sequenceFlow.setExecutionListeners(this.buidEventListener());
        sequenceFlow.setTargetRef(
                Optional.ofNullable(this.getNext()).map(Node::getId).orElse(this.getBranchId())
        );
        if (Objects.nonNull(conditions)) {
            String expression = this.toExpression(conditions);
            if (StringUtils.isNotBlank(expression)) {
                sequenceFlow.setConditionExpression(String.format("${%s}", expression));
            }
        }
        elements.add(sequenceFlow);
        // 下一个节点
        Node next = this.getNext();
        if (Objects.nonNull(next)) {
            next.setBranchId(this.getBranchId());
            List<FlowElement> flowElements = next.convert();
            elements.addAll(flowElements);
        }
        return elements;
    }
}
