package com.flow.entity.node;

import com.flow.common.core.exception.BaseException;
import com.flow.entity.node.connector.Connector;
import com.google.common.collect.Lists;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.flowable.bpmn.model.FlowElement;
import org.flowable.bpmn.model.SequenceFlow;
import org.flowable.bpmn.model.ServiceTask;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@EqualsAndHashCode(callSuper = true)
@Data
public class ConnectorNode extends Node {
    private Connector connector;

    @Override
    public List<FlowElement> convert() {
        ArrayList<FlowElement> elements = Lists.newArrayList();
        // 服务节点
        ServiceTask serviceTask = new ServiceTask();
        serviceTask.setId(this.getId());
        serviceTask.setName(this.getName());
        serviceTask.setExecutionListeners(this.buidEventListener());
        if(Objects.isNull(this.connector)){
            throw new BaseException("连接器不能为空");
        }
        this.connector.initImpl(serviceTask);
        elements.add(serviceTask);
        // 下一个节点的连线
        Node next = this.getNext();
        SequenceFlow sequenceFlow = this.buildSequence(next);
        elements.add(sequenceFlow);
        // 下一个节点
        if (Objects.nonNull(next)) {
            next.setBranchId(this.getBranchId());
            List<FlowElement> flowElements = next.convert();
            elements.addAll(flowElements);
        }
        return elements;
    }
}
