package com.flow.entity.node;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.flow.common.core.util.ApplicationContextUtil;
import com.google.common.collect.Lists;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.flowable.bpmn.model.*;

import java.util.*;

@EqualsAndHashCode(callSuper = true)
@Data
public class HttpNode extends Node {
    private String url;
    private String method;
    private List<Map<String, Object>> headers;
    private List<Map<String, Object>> body;
    private Long requestTimeout = 0L;
    private List<String> errorUsers;

    @Override
    public List<FlowElement> convert() {
        ArrayList<FlowElement> elements = Lists.newArrayList();
        HttpServiceTask httpServiceTask = new HttpServiceTask();
        httpServiceTask.setId(this.getId());
        httpServiceTask.setType("http");
        httpServiceTask.setName(this.getName());
        httpServiceTask.setExecutionListeners(this.buidEventListener());
        // httpServiceTask.setAsynchronous(true);
        // httpServiceTask.setParallelInSameTransaction(true);
        // 失败重试
        /*boolean isRetry = Objects.nonNull(this.retryCount) && this.retryCount > 0;
        if (isRetry) {
            StringBuilder retry = new StringBuilder();
            retry.append(String.format("R%s", this.retryCount));
            if (Objects.nonNull(this.duration)) {
                retry.append("/")
                        .append(String.format(this.unit.getUnit(), this.duration));
                httpServiceTask.setFailedJobRetryTimeCycleValue(retry.toString());
            }
        }*/
        // 请求参数
        ArrayList<FieldExtension> fieldExtensions = new ArrayList<>();
        FieldExtension urlExtension = new FieldExtension();
        urlExtension.setFieldName("requestUrl");
        urlExtension.setExpression(this.url);
        fieldExtensions.add(urlExtension);
        FieldExtension methodExtension = new FieldExtension();
        methodExtension.setFieldName("requestMethod");
        methodExtension.setStringValue(this.method);
        fieldExtensions.add(methodExtension);
        try {
            ObjectMapper objectMapper = ApplicationContextUtil.getBean(ObjectMapper.class);
            if (CollectionUtils.isNotEmpty(this.headers)) {
                Map<Object, Object> map = new HashMap<>();
                for (Map<String, Object> header : this.headers) {
                    Object label = header.get("label");
                    Object value = header.get("value");
                    if (Objects.nonNull(label) && Objects.nonNull(value)) {
                        map.put(label, value);
                    }
                }
                String headers = objectMapper.writeValueAsString(map);
                FieldExtension headerExtension = new FieldExtension();
                headerExtension.setFieldName("requestHeaders");
                headerExtension.setExpression(headers);
                fieldExtensions.add(headerExtension);
            }
            if (CollectionUtils.isNotEmpty(this.body)) {
                Map<Object, Object> map = new HashMap<>();
                for (Map<String, Object> boy : this.body) {
                    Object label = boy.get("label");
                    Object value = boy.get("value");
                    if (Objects.nonNull(label) && Objects.nonNull(value)) {
                        map.put(label, value);
                    }
                }
                String body = objectMapper.writeValueAsString(map);
                FieldExtension bodyExtension = new FieldExtension();
                bodyExtension.setFieldName("requestBody");
                bodyExtension.setExpression(body);
                fieldExtensions.add(bodyExtension);
            }
        } catch (JsonProcessingException ignored) {
        }
        FieldExtension timeoutExtension = new FieldExtension();
        timeoutExtension.setFieldName("requestTimeout");
        timeoutExtension.setStringValue(this.requestTimeout.toString());
        fieldExtensions.add(timeoutExtension);
        FieldExtension failCodesExtension = new FieldExtension();
        failCodesExtension.setFieldName("failStatusCodes");
        failCodesExtension.setStringValue("400,401,403,404,500,503");
        fieldExtensions.add(failCodesExtension);
        FieldExtension handleCodesExtension = new FieldExtension();
        handleCodesExtension.setFieldName("handleStatusCodes");
        handleCodesExtension.setStringValue("400,401,403,404,500,503");
        fieldExtensions.add(handleCodesExtension);
        FieldExtension saveResponseBodyExtension = new FieldExtension();
        saveResponseBodyExtension.setFieldName("saveResponseVariableAsJson");
        saveResponseBodyExtension.setStringValue("true");
        fieldExtensions.add(saveResponseBodyExtension);
        FieldExtension activityBehaviorExtension = new FieldExtension();
        activityBehaviorExtension.setFieldName("httpActivityBehaviorClass");
        activityBehaviorExtension.setStringValue("com.flow.behavior.CustomHttpServiceTaskActivityBehavior");
        fieldExtensions.add(activityBehaviorExtension);
        httpServiceTask.setFieldExtensions(fieldExtensions);
        FieldExtension ignoreExtension = new FieldExtension();
        ignoreExtension.setFieldName("ignoreException");
        ignoreExtension.setStringValue(Boolean.toString(false));
        fieldExtensions.add(ignoreExtension);
        elements.add(httpServiceTask);
        elements.addAll(this.errorHandler(httpServiceTask));
        // 下一个节点的连线
        Node next = this.getNext();
        SequenceFlow sequenceFlow = this.buildSequence(next);
        elements.add(sequenceFlow);
        // 下一个节点
        if (Objects.nonNull(next)) {
            next.setBranchId(this.getBranchId());
            List<FlowElement> flowElements = next.convert();
            elements.addAll(flowElements);
        }
        return elements;
    }

    private List<FlowElement> errorHandler(Activity activity) {
        ArrayList<FlowElement> elements = Lists.newArrayList();
        // 错误边界事件
        BoundaryEvent boundaryEvent = new BoundaryEvent();
        String boundaryId = String.format("%s_boundary", activity.getId());
        boundaryEvent.setId(boundaryId);
        boundaryEvent.setName("错误处理");
        boundaryEvent.setCancelActivity(true);
        boundaryEvent.setAttachedToRef(activity);
        ErrorEventDefinition errorEventDefinition = new ErrorEventDefinition();
        // errorEventDefinition.setErrorCode("HTTP400");
        boundaryEvent.setEventDefinitions(Lists.newArrayList(errorEventDefinition));
        elements.add(boundaryEvent);
        // 连接线
        String targetRef = String.format("%s_error", activity.getId());
        SequenceFlow sequenceFlow = new SequenceFlow();
        sequenceFlow.setId(String.format("%s-%s", boundaryId, targetRef));
        sequenceFlow.setSourceRef(boundaryId);
        sequenceFlow.setTargetRef(targetRef);
        elements.add(sequenceFlow);
        // 人工处理
        UserTask userTask = new UserTask();
        userTask.setId(targetRef);
        userTask.setName(String.format("%s(%s)", activity.getName(), "错误处理"));
        MultiInstanceLoopCharacteristics multiInstanceLoopCharacteristics = new MultiInstanceLoopCharacteristics();
        multiInstanceLoopCharacteristics.setSequential(false);
        String variable = String.format("%sItem", targetRef);
        multiInstanceLoopCharacteristics.setElementVariable(variable);
        multiInstanceLoopCharacteristics.setInputDataItem(String.format("${%sCollection}", activity.getId()));
        userTask.setLoopCharacteristics(multiInstanceLoopCharacteristics);
        userTask.setAssignee(String.format("${%s}", variable));
        elements.add(userTask);
        // 连接线
        SequenceFlow sequenceNext = new SequenceFlow();
        sequenceNext.setId(String.format("%s-%s", targetRef, activity.getId()));
        sequenceNext.setSourceRef(targetRef);
        sequenceNext.setTargetRef(activity.getId());
        elements.add(sequenceNext);
        return elements;
    }
}
