package com.flow.entity.node;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.flow.common.mybatis.entity.FilterGroup;
import com.flow.flowable.model.JumpServiceTask;
import com.google.common.collect.Lists;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.flowable.bpmn.model.FieldExtension;
import org.flowable.bpmn.model.FlowElement;
import org.flowable.bpmn.model.SequenceFlow;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@EqualsAndHashCode(callSuper = true)
@Data
public class JumpNode extends Node {
    private String targetNode;
    private FilterGroup conditions;

    @Override
    public List<FlowElement> convert() {
        ArrayList<FlowElement> elements = Lists.newArrayList();
        // 服务节点
        JumpServiceTask jumpServiceTask = new JumpServiceTask();
        jumpServiceTask.setId(this.getId());
        jumpServiceTask.setType("jump");
        jumpServiceTask.setName(this.getName());
        jumpServiceTask.setExecutionListeners(this.buidEventListener());
        // jumpServiceTask.setImplementationType(ImplementationType.IMPLEMENTATION_TYPE_DELEGATEEXPRESSION);
        // jumpServiceTask.setImplementation("${jumpDelegate}");
        // 扩展属性
        FieldExtension fieldExtension = new FieldExtension();
        fieldExtension.setFieldName("targetNode");
        fieldExtension.setStringValue(this.targetNode);
        jumpServiceTask.getFieldExtensions().add(fieldExtension);
        if (Objects.nonNull(this.conditions)) {
            String expression = ConditionNode.toExpression(this.conditions);
            if (StringUtils.isNotBlank(expression)) {
                jumpServiceTask.setSkipExpression(String.format("${!(%s)}", expression));
            }
        }
        elements.add(jumpServiceTask);
        // 下一个节点的连线
        Node next = this.getNext();
        SequenceFlow sequenceFlow = this.buildSequence(next);
        elements.add(sequenceFlow);
        // 下一个节点
        if (Objects.nonNull(next)) {
            next.setBranchId(this.getBranchId());
            List<FlowElement> flowElements = next.convert();
            elements.addAll(flowElements);
        }
        return elements;
    }
}
