package com.flow.entity.node;

import com.flow.enums.NotifyTypeEnum;
import com.google.common.collect.Lists;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.flowable.bpmn.model.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@EqualsAndHashCode(callSuper = true)
@Data
public class NotifyNode extends AssigneeNode {
    private List<NotifyTypeEnum> types;
    private String subject;
    private String content;

    @Override
    public List<FlowElement> convert() {
        ArrayList<FlowElement> elements = Lists.newArrayList();
        // 服务节点
        ServiceTask serviceTask = new ServiceTask();
        serviceTask.setId(this.getId());
        serviceTask.setName(this.getName());
        serviceTask.setExecutionListeners(this.buidEventListener());
        serviceTask.setAsynchronous(true);
        serviceTask.setImplementationType(ImplementationType.IMPLEMENTATION_TYPE_DELEGATEEXPRESSION);
        serviceTask.setImplementation("${notifyDelegate}");
        String variable = String.format("%sItem", this.getId());
        MultiInstanceLoopCharacteristics multiInstanceLoopCharacteristics = new MultiInstanceLoopCharacteristics();
        multiInstanceLoopCharacteristics.setSequential(false);
        multiInstanceLoopCharacteristics.setElementVariable(variable);
        multiInstanceLoopCharacteristics.setInputDataItem(String.format("${%sCollection}", this.getId()));
        serviceTask.setLoopCharacteristics(multiInstanceLoopCharacteristics);
        // 扩展属性
        FieldExtension typeExtension = new FieldExtension();
        typeExtension.setFieldName("types");
        String types = this.types.stream().map(NotifyTypeEnum::getType).collect(Collectors.joining(","));
        typeExtension.setStringValue(types);
        serviceTask.getFieldExtensions().add(typeExtension);
        FieldExtension subjectExtension = new FieldExtension();
        subjectExtension.setFieldName("subject");
        subjectExtension.setExpression(this.subject);
        serviceTask.getFieldExtensions().add(subjectExtension);
        FieldExtension contentExtension = new FieldExtension();
        contentExtension.setFieldName("content");
        contentExtension.setExpression(this.content);
        serviceTask.getFieldExtensions().add(contentExtension);
        elements.add(serviceTask);
        // 下一个节点的连线
        Node next = this.getNext();
        SequenceFlow sequenceFlow = this.buildSequence(next);
        elements.add(sequenceFlow);
        // 下一个节点
        if (Objects.nonNull(next)) {
            next.setBranchId(this.getBranchId());
            List<FlowElement> flowElements = next.convert();
            elements.addAll(flowElements);
        }
        return elements;
    }
}
