package com.flow.entity.node;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.google.common.collect.Lists;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.flowable.bpmn.model.FlowElement;
import org.flowable.bpmn.model.InclusiveGateway;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@EqualsAndHashCode(callSuper = true)
@Data
public class ParallelNode extends BranchNode {
    private String name;

    @Override
    public List<FlowElement> convert() {
        ArrayList<FlowElement> elements = Lists.newArrayList();
        // 并行分支
        InclusiveGateway inclusiveGateway = new InclusiveGateway();
        inclusiveGateway.setId(this.getId());
        inclusiveGateway.setName(this.getName());
        inclusiveGateway.setExecutionListeners(this.buidEventListener());
        elements.add(inclusiveGateway);
        List<ConditionNode> branches = this.getBranches();
        // 分支聚合
        InclusiveGateway gateway = new InclusiveGateway();
        gateway.setId(String.format("%send", this.getId()));
        gateway.setName("并行聚合");
        elements.addAll(this.merge(gateway));
        // 子节点
        if (CollectionUtils.isNotEmpty(branches)) {
            for (Node next : branches) {
                next.setBranchId(gateway.getId());
                elements.addAll(next.convert());
            }
        }
        // 下一个节点
        Node next = this.getNext();
        if (Objects.nonNull(next)) {
            next.setBranchId(this.getBranchId());
            List<FlowElement> flowElements = next.convert();
            elements.addAll(flowElements);
        }
        return elements;
    }
}
