package com.flow.model;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.flow.entity.FlowDefine;
import com.flow.entity.FormModel;
import com.flow.entity.node.FieldProperty;
import com.flow.enums.ProcessStatus;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.time.LocalDateTime;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

@Data
@AllArgsConstructor
@NoArgsConstructor
public class FormInfo {
    // 表单名称
    private String formName;
    // 流程实例id
    private String instanceId;
    // 任务id
    private String taskId;
    // 流程定义
    private FlowDefine flowDefine;
    // 状态
    private ProcessStatus status;
    // 发起人
    private String startUserId;
    // 撤销天数
    private Integer cancelDays;
    // 是否可撤销
    private Boolean isCancel = false;
    // 发起时间
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime startTime;
    // 结束时间
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime endTime;
    // 操作权限
    private Map<String, Boolean> operations = new LinkedHashMap<>();

    public FormInfo(FlowDefine flowDefine) {
        this.flowDefine = flowDefine;
        if (Objects.nonNull(flowDefine)) {
            this.formName = flowDefine.getName();
        }
    }

    public FormInfo(FlowDefine flowDefine,
                    List<FieldProperty> formProperties,
                    Map<String, Object> variables) {
        this(flowDefine);
        if (Objects.nonNull(flowDefine)) {
            FormModel formModel = flowDefine.getForm();
            formModel.initialization(formProperties);
            formModel.initialization(variables);
        }
    }

    public FormInfo(FlowDefine flowDefine,
                    List<FieldProperty> formProperties,
                    Map<String, Object> variables,
                    Map<String, Boolean> operations) {
        this(flowDefine, formProperties, variables);
        this.operations = operations;
    }

}
